import itertools

import yaml
from yaml.dumper import Emitter, Serializer, Representer, Resolver
from yaml.loader import Reader, Scanner, Composer, Constructor, Resolver
from yaml.emitter import DocumentStartEvent, StreamEndEvent, EmitterError
from yaml.parser import Parser
from yaml.nodes import Node, MappingNode, SequenceNode, ScalarNode


class DummyRepresenter(Representer):
    def __init__(self):
        Representer.__init__(self)

    def represent_data(self, data):
        return data


class ImplicitEndEmitter(Emitter):
    def __init__(self, stream, canonical=None, indent=None, width=None,
                 allow_unicode=None, line_break=None):
        Emitter.__init__(self, stream, canonical=canonical,
                         indent=indent, width=width,
                         allow_unicode=allow_unicode, line_break=line_break)

    def expect_document_start(self, first=False):
        if isinstance(self.event, DocumentStartEvent):
            Emitter.expect_document_start(self, first=first)
        elif isinstance(self.event, StreamEndEvent):
            self.write_stream_end()
            self.state = self.expect_nothing
        else:
            raise EmitterError("expected DocumentStartEvent or StreamEndEvent, but got %s"
                               % self.event)


class AnchorSerializer(Serializer):
    def __init__(self, encoding=None,
                 explicit_start=True, explicit_end=False, version=None, tags=None):
        Serializer.__init__(self, encoding=encoding,
                            explicit_start=explicit_start, explicit_end=False,
                            version=version, tags=tags)

    def generate_anchor(self, node):
        return node.anchor


class AnchorComposer(Composer):
    def __init__(self):
        Composer.__init__(self)

    def compose_scalar_node(self, anchor):
        node = Composer.compose_scalar_node(self, anchor)
        node.anchor = anchor
        return node

    def compose_mapping_node(self, anchor):
        node = Composer.compose_mapping_node(self, anchor)
        node.anchor = anchor
        return node

    def compose_sequence_node(self, anchor):
        node = Composer.compose_sequence_node(self, anchor)
        node.anchor = anchor
        return node


class NamedAnchorsLoader(Reader, Scanner, Parser, AnchorComposer, Constructor, Resolver):
    def __init__(self, stream):
        Reader.__init__(self, stream)
        Scanner.__init__(self)
        Parser.__init__(self)
        AnchorComposer.__init__(self)
        Constructor.__init__(self)
        Resolver.__init__(self)


class NodeDumper(ImplicitEndEmitter, AnchorSerializer, DummyRepresenter, Resolver):

    def __init__(self, stream, default_style=None, default_flow_style=None,
                 canonical=None, indent=None, width=None,
                 allow_unicode=None, line_break=None,
                 encoding=None, explicit_start=None, explicit_end=None,
                 version=None, tags=None, sort_keys=True):
        ImplicitEndEmitter.__init__(self, stream, canonical=canonical,
                                    indent=indent, width=width,
                                    allow_unicode=allow_unicode, line_break=line_break)
        AnchorSerializer.__init__(self, encoding=encoding,
                                  explicit_start=explicit_start, explicit_end=False,
                                  version=version, tags=tags)
        DummyRepresenter.__init__(self)
        Resolver.__init__(self)


def compose_all(input):
    documents_iterable = yaml.compose_all(stream=input, Loader=NamedAnchorsLoader)
    try:
        first = next(documents_iterable)
    except StopIteration:
        return yaml.compose_all('""', Loader=NamedAnchorsLoader)
    return itertools.chain([first], documents_iterable)


def dump_all(nodes):
    return yaml.dump_all(nodes, explicit_start=True, Dumper=NodeDumper)


def compose(input):
    result = yaml.compose(stream=input, Loader=NamedAnchorsLoader)
    return result if result is not None else yaml.compose('""', Loader=NamedAnchorsLoader)


def dump(nodes):
    return yaml.dump(nodes, explicit_start=False, Dumper=NodeDumper)


def equals(node1, node2):
    if type(node1) != type(node2):
        return False

    if node1.tag != node2.tag:
        return False

    if isinstance(node1, MappingNode):
        return mapping_equals(node1, node2)
    if isinstance(node1, SequenceNode):
        return sequence_equals(node1, node2)
    if isinstance(node1, ScalarNode):
        return scalar_equals(node1, node2)
    raise ValueError('Expected node of type MappingNode, SequenceNode or ScalarNode, but got {}'.format(type(node1)))


def scalar_equals(node1, node2):
    return node1.value == node2.value


def mapping_equals(node1, node2):
    if len(node1.value) != len(node2.value):
        return False

    map1 = dict(node1.value)
    map2 = dict(node2.value)

    shared_keys = []

    for key1 in map1.keys():
        for key2 in map2.keys():
            if equals(key1, key2):
                shared_keys.append(tuple([key1, key2]))
                break

    if len(shared_keys) != len(map1):
        return False

    for key1, key2 in shared_keys:
        if not equals(map1[key1], map2[key2]):
            return False

    return True


def sequence_equals(node1, node2):
    if len(node1.value) != len(node2.value):
        return False

    for item1, item2 in zip(node1.value, node2.value):
        if not equals(item1, item2):
            return False

    return True



from SpirentSLC.yaml_execution.matcher import YamlQuery, match
from SpirentSLC.yaml_execution.utils import compose, compose_all, dump, dump_all
from SpirentSLC.yaml_execution.matcher import MappingParentMatch, DocumentStreamMatch, SequenceParentMatch
from SpirentSLC.yaml_execution.errors import *
from yaml.nodes import MappingNode, SequenceNode, ScalarNode
from yaml.error import YAMLError


class YamlOperation:

    def __init__(self, query):
        self.query = query

    def compose_source(self, source):
        try:
            return list(compose_all(source))
        except YAMLError:
            raise InvalidSourceError('Failed to parse source YAML')


class YamlOperationWithValue(YamlOperation):
    def compose_value(self):
        try:
            return compose(self.value)
        except YAMLError:
            raise InvalidValueError('Failed to parse insertion value')

    def __init__(self, query, value):
        YamlOperation.__init__(self, query)
        self.value = value


class YamlGetOperation(YamlOperation):

    def __init__(self, query):
        YamlOperation.__init__(self, query)

    def execute(self, input):
        nodes = self.compose_source(input)
        matched_node = match(nodes, self.query)
        return dump(matched_node.value)


class YamlDeleteOperation(YamlOperation):

    def __init__(self, query):
        YamlOperation.__init__(self, query)

    def execute(self, input):
        nodes = self.compose_source(input)
        self.delete_node(match(nodes, self.query))
        return dump_all(nodes)

    def delete_node(self, match):
        if isinstance(match, DocumentStreamMatch):
            del match.documents[match.index]
        if isinstance(match, SequenceParentMatch):
            del match.sequence.value[match.index]
        if isinstance(match, MappingParentMatch):
            del match.mapping.value[match.index]


class YamlSetOperation(YamlOperationWithValue):

    def __init__(self, query, value):
        YamlOperationWithValue.__init__(self, query, value)

    def execute(self, input):
        nodes = self.compose_source(input)
        self.replace_node(match(nodes, self.query))
        return dump_all(nodes)

    def replace_node(self, match):
        new_node = self.compose_value()
        if isinstance(match, DocumentStreamMatch):
            match.documents[match.index] = new_node
        if isinstance(match, SequenceParentMatch):
            match.sequence.value[match.index] = new_node
        if isinstance(match, MappingParentMatch):
            match.mapping.value[match.index] = (match.entry[0], new_node)


class YamlAddOperation(YamlOperationWithValue):

    def __init__(self, query, value):
        YamlOperationWithValue.__init__(self, query, value)

    def compose_key(self):
        try:
            return compose(self.query.segments[-1])
        except YAMLError:
            raise InvalidQueryError('Failed to parse insertion key')

    def execute(self, input):
        nodes = self.compose_source(input)

        if len(self.query.segments) == 0:
            index = self.query.document_index
            if index < 0 or index > len(nodes):
                raise InvalidQueryError(
                    'Index path segment should be in range [{}, {}]'.format(0, len(nodes)))
            new_node = self.compose_value()
            nodes.insert(index, new_node)
        else:
            self.add_node(nodes)

        return dump_all(nodes)

    def add_node(self, nodes):
        # Match parent
        parent_query = YamlQuery(self.query.document_index, self.query.segments[:-1])
        parent_match = match(nodes, parent_query)
        container_node = parent_match.value

        if isinstance(container_node, MappingNode):
            key_segment = self.query.segments[-1]
            try:
                match([container_node], YamlQuery(0, [key_segment]))
                raise KeyExistsError('A mapping entry with the key \'{}\' already exists'.format(key_segment))
            except NotFoundError:
                new_key = self.compose_key()
                new_node = self.compose_value()
                container_node.value.append(tuple([new_key, new_node]))
                return

        if isinstance(container_node, SequenceNode):
            index_segment = self.query.segments[-1]
            try:
                index = int(index_segment)
            except ValueError:
                raise InvalidQueryError('Index path segment \'{}\' cannot be parsed to integer'.format(index_segment))
            if index < 0 or index > len(container_node.value):
                raise InvalidQueryError(
                    'Index path segment should be in range [{}, {}]'.format(0, len(self.query.segments)))
            new_node = self.compose_value()
            container_node.value.insert(index, new_node)
            return

        if isinstance(container_node, ScalarNode):
            raise InvalidQueryError('Can only add values into maps and sequences, but not scalars')

        raise ValueError('Unexpected type of node: {}'.format(type(container_node)))



class NotFoundError(Exception):
    pass


class InvalidQueryError(Exception):
    pass


class KeyExistsError(Exception):
    pass


class InvalidValueError(Exception):
    pass


class InvalidSourceError(Exception):
    pass



import os.path

resouces_path = os.path.dirname(os.path.abspath(__file__))
repo_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'itars')
data_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ExecutionProtocol.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x45xecutionProtocol.proto\x12&com.spirent.service.execution.protocol\"\xbf\x1d\n\x10\x45xecutionMessage\x12K\n\x04type\x18\x01 \x02(\x0e\x32=.com.spirent.service.execution.protocol.ExecutionMessage.Type\x12L\n\rcancelMessage\x18\x02 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.CancelMessage\x12R\n\x10heartbeatMessage\x18\x03 \x01(\x0b\x32\x38.com.spirent.service.execution.protocol.HeartbeatMessage\x12H\n\x0binitMessage\x18\x04 \x01(\x0b\x32\x33.com.spirent.service.execution.protocol.InitMessage\x12N\n\x0einitMessageAck\x18# \x01(\x0b\x32\x36.com.spirent.service.execution.protocol.InitMessageAck\x12\x44\n\tinvokeAck\x18\x05 \x01(\x0b\x32\x31.com.spirent.service.execution.protocol.InvokeAck\x12L\n\rinvokeMessage\x18\x06 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.InvokeMessage\x12N\n\x0einvokeResponse\x18\x07 \x01(\x0b\x32\x36.com.spirent.service.execution.protocol.InvokeResponse\x12L\n\rreportMessage\x18\x08 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.ReportMessage\x12H\n\x0bstepMessage\x18\t \x01(\x0b\x32\x33.com.spirent.service.execution.protocol.StepMessage\x12R\n\x10terminateMessage\x18\n \x01(\x0b\x32\x38.com.spirent.service.execution.protocol.TerminateMessage\x12J\n\x0ctokenMessage\x18\x0b \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.TokenMessage\x12\\\n\x15\x65xecutionIssueMessage\x18\x0c \x01(\x0b\x32=.com.spirent.service.execution.protocol.ExecutionIssueMessage\x12V\n\x12\x61gentStatusMessage\x18\x1d \x01(\x0b\x32:.com.spirent.service.execution.protocol.AgentStatusMessage\x12Z\n\x14\x65xtractedDataMessage\x18  \x01(\x0b\x32<.com.spirent.service.execution.protocol.ExtractedDataMessage\x12M\n\x11newSessionMessage\x18\r \x01(\x0b\x32\x32.com.spirent.service.execution.protocol.NewSession\x12V\n\x12newSessionResponse\x18\x0e \x01(\x0b\x32:.com.spirent.service.execution.protocol.NewSessionResponse\x12Y\n\x17sendTerminalDataMessage\x18\x0f \x01(\x0b\x32\x38.com.spirent.service.execution.protocol.SendTerminalData\x12\x62\n\x18sendTerminalDataResponse\x18\x10 \x01(\x0b\x32@.com.spirent.service.execution.protocol.SendTerminalDataResponse\x12S\n\x14sessionActionMessage\x18\x11 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.SessionAction\x12\\\n\x15sessionActionResponse\x18\x12 \x01(\x0b\x32=.com.spirent.service.execution.protocol.SessionActionResponse\x12J\n\x0clistProjects\x18\x13 \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.ListProjects\x12Z\n\x14listProjectsResponse\x18\x14 \x01(\x0b\x32<.com.spirent.service.execution.protocol.ListProjectsResponse\x12J\n\x0cqueryProject\x18\x15 \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.QueryProject\x12Z\n\x14queryProjectResponse\x18\x16 \x01(\x0b\x32<.com.spirent.service.execution.protocol.QueryProjectResponse\x12J\n\x0cquerySession\x18\x17 \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.QuerySession\x12Z\n\x14querySessionResponse\x18\x18 \x01(\x0b\x32<.com.spirent.service.execution.protocol.QuerySessionResponse\x12L\n\rqueryTestCase\x18\x19 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.QueryTestCase\x12\\\n\x15queryTestCaseResponse\x18\x1a \x01(\x0b\x32=.com.spirent.service.execution.protocol.QueryTestCaseResponse\x12T\n\x11testCaseProcedure\x18\x1b \x01(\x0b\x32\x39.com.spirent.service.execution.protocol.TestCaseProcedure\x12\x64\n\x19testCaseProcedureResponse\x18\x1c \x01(\x0b\x32\x41.com.spirent.service.execution.protocol.TestCaseProcedureResponse\x12T\n\x11sessionTerminated\x18\x1e \x01(\x0b\x32\x39.com.spirent.service.execution.protocol.SessionTerminated\x12Z\n\x14sessionActionCapture\x18\x1f \x01(\x0b\x32<.com.spirent.service.execution.protocol.SessionActionCapture\x12@\n\x07\x65ncrypt\x18! \x01(\x0b\x32/.com.spirent.service.execution.protocol.Encrypt\x12P\n\x0f\x65ncryptResponse\x18\" \x01(\x0b\x32\x37.com.spirent.service.execution.protocol.EncryptResponse\"\xb4\x06\n\x04Type\x12\x12\n\x0e\x43\x41NCEL_MESSAGE\x10\x01\x12\x15\n\x11HEARTBEAT_MESSAGE\x10\x02\x12\x10\n\x0cINIT_MESSAGE\x10\x03\x12\x14\n\x10INIT_MESSAGE_ACK\x10\"\x12\x0e\n\nINVOKE_ACK\x10\x04\x12\x12\n\x0eINVOKE_MESSAGE\x10\x05\x12\x13\n\x0fINVOKE_RESPONSE\x10\x06\x12\x12\n\x0eREPORT_MESSAGE\x10\x07\x12\x10\n\x0cSTEP_MESSAGE\x10\x08\x12\x15\n\x11TERMINATE_MESSAGE\x10\t\x12\x11\n\rTOKEN_MESSAGE\x10\n\x12\x1b\n\x17\x45XECUTION_ISSUE_MESSAGE\x10\x0b\x12\x18\n\x14\x41GENT_STATUS_MESSAGE\x10\x1c\x12\x1a\n\x16\x45XTRACTED_DATA_MESSAGE\x10\x1f\x12\x17\n\x13NEW_SESSION_MESSAGE\x10\x0c\x12\x18\n\x14NEW_SESSION_RESPONSE\x10\r\x12\x1e\n\x1aSEND_TERMINAL_DATA_MESSAGE\x10\x0e\x12\x1f\n\x1bSEND_TERMINAL_DATA_RESPONSE\x10\x0f\x12\x1a\n\x16SESSION_ACTION_MESSAGE\x10\x10\x12\x1b\n\x17SESSION_ACTION_RESPONSE\x10\x11\x12\x11\n\rLIST_PROJECTS\x10\x12\x12\x1a\n\x16LIST_PROJECTS_RESPONSE\x10\x13\x12\x11\n\rQUERY_PROJECT\x10\x14\x12\x1a\n\x16QUERY_PROJECT_RESPONSE\x10\x15\x12\x11\n\rQUERY_SESSION\x10\x16\x12\x1a\n\x16QUERY_SESSION_RESPONSE\x10\x17\x12\x13\n\x0fQUERY_TEST_CASE\x10\x18\x12\x1c\n\x18QUERY_TEST_CASE_RESPONSE\x10\x19\x12\x17\n\x13TEST_CASE_PROCEDURE\x10\x1a\x12 \n\x1cTEST_CASE_PROCEDURE_RESPONSE\x10\x1b\x12\x16\n\x12SESSION_TERMINATED\x10\x1d\x12\x1a\n\x16SESSION_ACTION_CAPTURE\x10\x1e\x12\x0b\n\x07\x45NCRYPT\x10 \x12\x14\n\x10\x45NCRYPT_RESPONSE\x10!\"}\n\x12\x41gentStatusMessage\x12\x0f\n\x07\x61gentId\x18\x01 \x02(\t\x12\x14\n\x0clastActivity\x18\x02 \x01(\x04\x12\x13\n\x0bonlineSince\x18\x03 \x01(\x04\x12\x11\n\tsessionId\x18\x04 \x01(\t\x12\x18\n\x10unresponsiveTime\x18\x05 \x01(\x04\"\"\n\rCancelMessage\x12\x11\n\tsessionId\x18\x01 \x02(\t\"\x9e\x04\n\x15\x45xecutionIssueMessage\x12\x0f\n\x07\x66ileUri\x18\x01 \x02(\t\x12\x12\n\nissueIndex\x18\x02 \x02(\r\x12\x10\n\x08location\x18\x03 \x02(\t\x12\x0f\n\x07message\x18\x04 \x01(\t\x12\x12\n\noriginator\x18\x05 \x01(\t\x12\x15\n\rprocedureName\x18\x06 \x01(\t\x12\x10\n\x08reportId\x18\x07 \x02(\t\x12\x11\n\tsessionId\x18\x08 \x02(\t\x12\x65\n\x08severity\x18\t \x01(\x0e\x32\x46.com.spirent.service.execution.protocol.ExecutionIssueMessage.Severity:\x0bINFORMATION\x12\x0e\n\x06stepId\x18\n \x02(\t\x12\x16\n\x0etestCaseStepId\x18\x0b \x02(\t\x12\x0c\n\x04line\x18\x0c \x01(\x04\x12\x10\n\x08startCol\x18\r \x01(\x04\x12\x0e\n\x06\x65ndCol\x18\x0e \x01(\x04\x12\x11\n\tlineCount\x18\x0f \x01(\x04\x12\x13\n\x0bsequenceNum\x18\x10 \x01(\x04\x12\x11\n\tissueTime\x18\x11 \x01(\x04\x12\x1f\n\x10_slcSessionIssue\x18\x12 \x01(\x08:\x05\x66\x61lse\x12\x13\n\x04type\x18\x13 \x01(\t:\x05issue\"=\n\x08Severity\x12\t\n\x05\x45RROR\x10\x00\x12\x0f\n\x0bINFORMATION\x10\x01\x12\x08\n\x04PASS\x10\x02\x12\x0b\n\x07WARNING\x10\x03\"\xeb\x01\n\x14\x45xtractedDataMessage\x12\x10\n\x08reportId\x18\x01 \x02(\t\x12\x14\n\x0c\x65xecutedStep\x18\x02 \x01(\t\x12\x16\n\x0e\x65xecutableStep\x18\x03 \x01(\t\x12\x0c\n\x04time\x18\x04 \x01(\x01\x12\x0e\n\x06valNum\x18\x05 \x01(\x01\x12\x11\n\tvalNumAry\x18\x06 \x03(\x01\x12\x0e\n\x06valStr\x18\x07 \x01(\t\x12\x11\n\tvalStrAry\x18\x08 \x03(\t\x12\x0b\n\x03tag\x18\n \x02(\t\x12\x1a\n\x12\x64\x61taCollectionTime\x18\x0b \x02(\x04\x12\x16\n\x04type\x18\x0c \x01(\t:\x08\x65xtdata2\"b\n\x10HeartbeatMessage\x12\x17\n\x0bmemoryUsage\x18\x01 \x01(\x01:\x02-1\x12\x1a\n\x0eprocessCpuLoad\x18\x02 \x01(\x01:\x02-1\x12\x19\n\rsystemCpuLoad\x18\x03 \x01(\x01:\x02-1\"1\n\tInvokeAck\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x11\n\tstartTime\x18\x02 \x01(\x04\"\xa1\x01\n\x05Param\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\x41\n\nparameters\x18\x03 \x03(\x0b\x32-.com.spirent.service.execution.protocol.Param\x12\x15\n\x06masked\x18\x04 \x01(\x08:\x05\x66\x61lse\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\"\x9e\x06\n\rInvokeMessage\x12\x43\n\x05token\x18\x01 \x02(\x0b\x32\x34.com.spirent.service.execution.protocol.TokenMessage\x12L\n\x06header\x18\x02 \x03(\x0b\x32<.com.spirent.service.execution.protocol.InvokeMessage.Header\x12\x10\n\x08reportId\x18\x03 \x02(\t\x12\x15\n\rdependencyUri\x18\x04 \x03(\t\x12\x14\n\x0cparamFileUri\x18\x05 \x03(\t\x12\x41\n\nparameters\x18\x06 \x03(\x0b\x32-.com.spirent.service.execution.protocol.Param\x12V\n\x0breservation\x18\x07 \x01(\x0b\x32\x41.com.spirent.service.execution.protocol.InvokeMessage.Reservation\x12\x12\n\ntestbedUri\x18\x08 \x01(\t\x12\x13\n\x0btestCaseUri\x18\t \x02(\t\x12l\n\x0b\x64\x65tailLevel\x18\n \x01(\x0e\x32\x41.com.spirent.service.execution.protocol.InvokeMessage.DetailLevel:\x14\x41LL_ISSUES_ALL_STEPS\x1a%\n\x06Header\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\x1a\x44\n\x0bReservation\x12\x15\n\rreservationId\x18\x01 \x02(\t\x12\x1e\n\x16reservationVelocityUrl\x18\x02 \x02(\t\"\x9b\x01\n\x0b\x44\x65tailLevel\x12\x0f\n\x0bREPORT_ONLY\x10\x00\x12\x15\n\x11\x45RROR_ISSUES_ONLY\x10\x01\x12\x1b\n\x17\x45RROR_ISSUES_WITH_STEPS\x10\x02\x12\x1a\n\x16\x41LL_ISSUES_ERROR_STEPS\x10\x03\x12\x18\n\x14\x41LL_ISSUES_ALL_STEPS\x10\x04\x12\x11\n\rLAST_RESPONSE\x10\x05\"\xf7\x02\n\x0bInitMessage\x12\x0f\n\x07\x61gentId\x18\x01 \x02(\t\x12\x11\n\tagentName\x18\x02 \x02(\t\x12\x1e\n\x0fprotocolVersion\x18\x03 \x02(\t:\x05\x39.0.0\x12R\n\ncapability\x18\x04 \x03(\x0b\x32>.com.spirent.service.execution.protocol.InitMessage.Capability\x12\x11\n\tsessionId\x18\x05 \x01(\t\x12T\n\x0brestriction\x18\x06 \x03(\x0b\x32?.com.spirent.service.execution.protocol.InitMessage.Restriction\x12\x10\n\x08\x61gentKey\x18\x07 \x01(\t\x1a)\n\nCapability\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\x1a*\n\x0bRestriction\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"\xc1\x01\n\x0eInitMessageAck\x12\x0f\n\x07message\x18\x01 \x02(\t\x12]\n\x0e\x63onfigSettings\x18\x02 \x03(\x0b\x32\x45.com.spirent.service.execution.protocol.InitMessageAck.ConfigSettings\x12\x10\n\x08\x61gentKey\x18\x03 \x01(\t\x1a-\n\x0e\x43onfigSettings\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"\xa4\x02\n\x0eInvokeResponse\x12J\n\x0creportResult\x18\x01 \x02(\x0e\x32\x34.com.spirent.service.execution.protocol.ReportResult\x12\x11\n\tstartTime\x18\x02 \x02(\x04\x12\x0f\n\x07\x65ndTime\x18\x03 \x02(\x04\x12\x11\n\tsessionId\x18\x04 \x02(\t\x12K\n\x05\x65rror\x18\x05 \x01(\x0b\x32<.com.spirent.service.execution.protocol.InvokeResponse.Error\x12\x1a\n\x0bisDuplicate\x18\x06 \x01(\x08:\x05\x66\x61lse\x1a&\n\x05\x45rror\x12\x0c\n\x04\x63ode\x18\x01 \x02(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\"!\n\x10TerminateMessage\x12\r\n\x05\x63\x61use\x18\x01 \x01(\t\"\x1d\n\x0cTokenMessage\x12\r\n\x05token\x18\x01 \x02(\t\"\xe2\x0f\n\rReportMessage\x12\x10\n\x08reportId\x18\x01 \x02(\t\x12\x11\n\tstartTime\x18\x02 \x01(\x04\x12\x0f\n\x07\x65ndTime\x18\x03 \x01(\x04\x12\x17\n\x0fparentStepIndex\x18\x04 \x01(\t\x12\x16\n\x0eparentReportId\x18\x05 \x01(\t\x12\x10\n\x08testPath\x18\x06 \x01(\t\x12\x0e\n\x06testId\x18\x07 \x01(\t\x12\x44\n\x06result\x18\x08 \x01(\x0e\x32\x34.com.spirent.service.execution.protocol.ReportResult\x12\x1a\n\x12percentageComplete\x18\t \x01(\x01\x12\x41\n\nparameters\x18\n \x03(\x0b\x32-.com.spirent.service.execution.protocol.Param\x12T\n\x06status\x18\x0b \x01(\x0e\x32\x44.com.spirent.service.execution.protocol.ReportMessage.ExecutionState\x12\x11\n\tagentHost\x18\x0c \x01(\t\x12\x0f\n\x07\x61gentId\x18\r \x01(\t\x12\x11\n\tagentName\x18\x0e \x01(\t\x12\x14\n\x0c\x64ispatchTime\x18\x0f \x01(\x04\x12)\n!executionLastUnsatisfiedCondition\x18\x10 \x01(\t\x12\x15\n\rfailureReason\x18\x11 \x01(\t\x12\x1a\n\x12isReservationBySvc\x18\x12 \x01(\x08\x12\r\n\x05token\x18\x13 \x01(\t\x12\x1d\n\x15topologyReservationId\x18\x14 \x01(\t\x12\x13\n\x0b\x63\x61llbackURL\x18\x15 \x01(\t\x12\x12\n\ncreateTime\x18\x16 \x01(\x04\x12l\n\x0b\x64\x65tailLevel\x18\x17 \x01(\x0e\x32\x41.com.spirent.service.execution.protocol.ReportMessage.DetailLevel:\x14\x41LL_ISSUES_ALL_STEPS\x12\x0e\n\x06\x64omain\x18\x18 \x01(\t\x12[\n\x13ignoredRequirements\x18\x19 \x03(\x0b\x32>.com.spirent.service.execution.protocol.ReportMessage.KeyValue\x12\x15\n\rdiscardReport\x18\x1a \x01(\x08\x12\x16\n\x0eisRunlistFirst\x18\x1b \x01(\x08\x12\x15\n\risRunlistLast\x18\x1c \x01(\x08\x12\x18\n\x10latestUpdateTime\x18\x1d \x01(\x04\x12Z\n\x12mergedRequirements\x18\x1e \x03(\x0b\x32>.com.spirent.service.execution.protocol.ReportMessage.KeyValue\x12\r\n\x05oldPk\x18\x1f \x01(\x04\x12\x19\n\x11parameterFilePath\x18  \x01(\t\x12!\n\x19reportFileCompressionType\x18! \x01(\t\x12\x18\n\x10reportFileSource\x18\" \x01(\t\x12\x14\n\x0creportFormat\x18# \x01(\t\x12T\n\x0crequirements\x18$ \x03(\x0b\x32>.com.spirent.service.execution.protocol.ReportMessage.KeyValue\x12\x15\n\rreservationId\x18% \x01(\t\x12\x14\n\x0ctestCategory\x18& \x01(\t\x12\x12\n\ntopologyId\x18\' \x01(\t\x12\x14\n\x0ctopologyPath\x18( \x01(\t\x12\x19\n\x11topologyVersionId\x18) \x01(\t\x12\x0e\n\x06userId\x18* \x01(\t\x12\x15\n\rlocalTopology\x18+ \x01(\t\x12\x11\n\tnamespace\x18, \x01(\t\x12\x14\n\x0ctestCaseName\x18- \x01(\t\x12\x12\n\nrepository\x18. \x01(\t\x12\x14\n\x0creportIdPath\x18/ \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x30 \x01(\t\x12\x10\n\x08headline\x18\x31 \x01(\t\x12\r\n\x05owner\x18\x32 \x01(\t\x12\x15\n\rreportVersion\x18\x33 \x01(\t\x12\x13\n\x0brunlistGuid\x18\x34 \x01(\t\x12\x15\n\rrunlistItemId\x18\x35 \x01(\t\x12\x13\n\x0brunlistPath\x18\x36 \x01(\t\x12\x14\n\x0ctopologyName\x18\x37 \x01(\t\x1a\'\n\x08KeyValue\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"\x89\x01\n\x0e\x45xecutionState\x12\x0b\n\x07\x41\x42ORTED\x10\x00\x12\x18\n\x14\x41GENT_NOT_RESPONDING\x10\x01\x12\r\n\tCOMPLETED\x10\x02\x12\x0f\n\x0b\x44ISPATCHING\x10\x03\x12\x0f\n\x0bIN_PROGRESS\x10\x04\x12\r\n\tNOT_BEGUN\x10\x05\x12\x10\n\x0cSTART_FAILED\x10\x06\"\x9b\x01\n\x0b\x44\x65tailLevel\x12\x0f\n\x0bREPORT_ONLY\x10\x00\x12\x15\n\x11\x45RROR_ISSUES_ONLY\x10\x01\x12\x1b\n\x17\x45RROR_ISSUES_WITH_STEPS\x10\x02\x12\x1a\n\x16\x41LL_ISSUES_ERROR_STEPS\x10\x03\x12\x18\n\x14\x41LL_ISSUES_ALL_STEPS\x10\x04\x12\x11\n\rLAST_RESPONSE\x10\x05\"\xd0\x0f\n\x0bStepMessage\x12\x10\n\x08reportId\x18\x01 \x02(\t\x12\x0e\n\x06stepId\x18\x02 \x02(\t\x12\x14\n\x0cparentStepId\x18\x03 \x01(\t\x12\x16\n\x0etestCaseStepId\x18\x04 \x02(\t\x12\x10\n\x08threadId\x18\x05 \x01(\x05\x12\x11\n\tsessionId\x18\x06 \x01(\t\x12\x13\n\x0bsessionType\x18\x07 \x01(\t\x12\x11\n\tprocedure\x18\x08 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\t \x01(\t\x12\x0f\n\x07\x63ontext\x18\n \x01(\t\x12\x0e\n\x06target\x18\x0b \x01(\t\x12\x0f\n\x07\x63ommand\x18\x0c \x01(\t\x12\x13\n\x0bstartOffset\x18\r \x01(\x01\x12\x10\n\x08\x64uration\x18\x0e \x01(\x01\x12\x17\n\x08\x65mulated\x18\x0f \x01(\x08:\x05\x66\x61lse\x12R\n\nstepResult\x18\x10 \x01(\x0e\x32>.com.spirent.service.execution.protocol.StepMessage.StepResult\x12X\n\x07\x64\x65tails\x18\x11 \x01(\x0b\x32G.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails\x12\x12\n\nproperties\x18\x12 \x01(\t\x12\x13\n\x0bsequenceNum\x18\x13 \x01(\x04\x12\x12\n\x04type\x18\x14 \x01(\t:\x04step\x1az\n\x06\x41\x63tion\x12\x0e\n\x06\x61\x63tion\x18\x01 \x02(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12K\n\x07\x61\x63tions\x18\x03 \x03(\x0b\x32:.com.spirent.service.execution.protocol.StepMessage.Action\x1a\xec\t\n\x13StepResponseDetails\x12\x1b\n\x0cisAutoMapped\x18\x01 \x01(\x08:\x05\x66\x61lse\x12\x0c\n\x04\x62ody\x18\x02 \x01(\t\x12R\n\x0epostProcessing\x18\x03 \x03(\x0b\x32:.com.spirent.service.execution.protocol.StepMessage.Action\x12k\n\ranalysisRules\x18\x04 \x03(\x0b\x32T.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.AnalysisRule\x12n\n\x0estructuredData\x18\x05 \x03(\x0b\x32V.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.StructuredData\x12^\n\x07\x61liases\x18\x06 \x03(\x0b\x32M.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.Alias\x1a\xf4\x01\n\x05\x41lias\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0bqueryFormat\x18\x02 \x01(\t\x12i\n\targuments\x18\x03 \x03(\x0b\x32V.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.Alias.Argument\x12\x19\n\x0cxPathVersion\x18\x04 \x01(\t:\x03\x31.0\x1a\x42\n\x08\x41rgument\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0bvaluesQuery\x18\x02 \x01(\t\x12\x13\n\x0binterpretAs\x18\x03 \x01(\t\x1a\xa8\x01\n\x0c\x41nalysisRule\x12\x11\n\textractor\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x10\n\x08whenTrue\x18\x03 \x01(\t\x12\x11\n\twhenFalse\x18\x04 \x01(\t\x12K\n\x07\x61\x63tions\x18\x05 \x03(\x0b\x32:.com.spirent.service.execution.protocol.StepMessage.Action\x1a\xf6\x02\n\x0eStructuredData\x12\x0b\n\x03map\x18\x08 \x02(\t\x12\x16\n\x0eresponseMapUri\x18\t \x01(\t\x12j\n\x05items\x18\n \x03(\x0b\x32[.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.StructuredData.Item\x1a\xd2\x01\n\x04Item\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04line\x18\x02 \x01(\x05\x12\x11\n\tlineCount\x18\x03 \x01(\x05\x12\x10\n\x08startCol\x18\x04 \x01(\x05\x12\x0e\n\x06\x65ndCol\x18\x05 \x01(\x05\x12\r\n\x05value\x18\x06 \x01(\t\x12j\n\x05items\x18\x07 \x03(\x0b\x32[.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.StructuredData.Item\"L\n\nStepResult\x12\x0b\n\x07\x41\x42ORTED\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\x12\x0b\n\x07SKIPPED\x10\x02\x12\x0b\n\x07SUCCESS\x10\x03\x12\x0b\n\x07TIMEOUT\x10\x04\"\xce\x12\n\x1b\x43lientSessionControlMessage\x12V\n\x04type\x18\x01 \x02(\x0e\x32H.com.spirent.service.execution.protocol.ClientSessionControlMessage.Type\x12\x41\n\x0binitMessage\x18\x02 \x01(\x0b\x32,.com.spirent.service.execution.protocol.Init\x12J\n\x0cinitResponse\x18\x03 \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.InitResponse\x12M\n\x11newSessionMessage\x18\x04 \x01(\x0b\x32\x32.com.spirent.service.execution.protocol.NewSession\x12V\n\x12newSessionResponse\x18\x05 \x01(\x0b\x32:.com.spirent.service.execution.protocol.NewSessionResponse\x12Y\n\x17sendTerminalDataMessage\x18\x06 \x01(\x0b\x32\x38.com.spirent.service.execution.protocol.SendTerminalData\x12\x62\n\x18sendTerminalDataResponse\x18\x07 \x01(\x0b\x32@.com.spirent.service.execution.protocol.SendTerminalDataResponse\x12S\n\x14sessionActionMessage\x18\x08 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.SessionAction\x12\\\n\x15sessionActionResponse\x18\t \x01(\x0b\x32=.com.spirent.service.execution.protocol.SessionActionResponse\x12[\n\x18sessionTerminatedMessage\x18\n \x01(\x0b\x32\x39.com.spirent.service.execution.protocol.SessionTerminated\x12Q\n\x13listProjectsMessage\x18\x0b \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.ListProjects\x12Z\n\x14listProjectsResponse\x18\x0c \x01(\x0b\x32<.com.spirent.service.execution.protocol.ListProjectsResponse\x12Q\n\x13queryProjectMessage\x18\r \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.QueryProject\x12Z\n\x14queryProjectResponse\x18\x0e \x01(\x0b\x32<.com.spirent.service.execution.protocol.QueryProjectResponse\x12J\n\x0cquerySession\x18\x0f \x01(\x0b\x32\x34.com.spirent.service.execution.protocol.QuerySession\x12Z\n\x14querySessionResponse\x18\x10 \x01(\x0b\x32<.com.spirent.service.execution.protocol.QuerySessionResponse\x12L\n\rqueryTestCase\x18\x11 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.QueryTestCase\x12\\\n\x15queryTestCaseResponse\x18\x12 \x01(\x0b\x32=.com.spirent.service.execution.protocol.QueryTestCaseResponse\x12T\n\x11testCaseProcedure\x18\x13 \x01(\x0b\x32\x39.com.spirent.service.execution.protocol.TestCaseProcedure\x12\x64\n\x19testCaseProcedureResponse\x18\x14 \x01(\x0b\x32\x41.com.spirent.service.execution.protocol.TestCaseProcedureResponse\x12Z\n\x14sessionActionCapture\x18\x15 \x01(\x0b\x32<.com.spirent.service.execution.protocol.SessionActionCapture\"\x87\x04\n\x04Type\x12\x10\n\x0cINIT_MESSAGE\x10\x01\x12\x11\n\rINIT_RESPONSE\x10\x02\x12\x17\n\x13NEW_SESSION_MESSAGE\x10\x03\x12\x18\n\x14NEW_SESSION_RESPONSE\x10\x04\x12\x1e\n\x1aSEND_TERMINAL_DATA_MESSAGE\x10\x05\x12\x1f\n\x1bSEND_TERMINAL_DATA_RESPONSE\x10\x06\x12\x1a\n\x16SESSION_ACTION_MESSAGE\x10\x07\x12\x1b\n\x17SESSION_ACTION_RESPONSE\x10\x08\x12\x16\n\x12SESSION_TERMINATED\x10\t\x12\x11\n\rLIST_PROJECTS\x10\n\x12\x1a\n\x16LIST_PROJECTS_RESPONSE\x10\x0b\x12\x11\n\rQUERY_PROJECT\x10\x0c\x12\x1a\n\x16QUERY_PROJECT_RESPONSE\x10\r\x12\x11\n\rQUERY_SESSION\x10\x0e\x12\x1a\n\x16QUERY_SESSION_RESPONSE\x10\x0f\x12\x13\n\x0fQUERY_TEST_CASE\x10\x10\x12\x1c\n\x18QUERY_TEST_CASE_RESPONSE\x10\x11\x12\x17\n\x13TEST_CASE_PROCEDURE\x10\x12\x12 \n\x1cTEST_CASE_PROCEDURE_RESPONSE\x10\x13\x12\x1a\n\x16SESSION_ACTION_CAPTURE\x10\x14\"&\n\x04Init\x12\r\n\x05token\x18\x01 \x02(\t\x12\x0f\n\x07version\x18\x02 \x02(\t\"8\n\x0cInitResponse\x12\x17\n\x06status\x18\x01 \x02(\t:\x07success\x12\x0f\n\x07version\x18\x02 \x02(\t\"\x0e\n\x0cListProjects\"(\n\x14ListProjectsResponse\x12\x10\n\x08projects\x18\x01 \x03(\t\"\xe9\x01\n\x0fPropertiesGroup\x12\x0c\n\x04name\x18\x01 \x02(\t\x12T\n\nproperties\x18\x02 \x03(\x0b\x32@.com.spirent.service.execution.protocol.PropertiesGroup.Property\x12I\n\x08\x63hildren\x18\x03 \x03(\x0b\x32\x37.com.spirent.service.execution.protocol.PropertiesGroup\x1a\'\n\x08Property\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"\xda\x03\n\nNewSession\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x12\n\nsessionUri\x18\x02 \x02(\t\x12\x15\n\rdependencyUri\x18\x03 \x03(\t\x12I\n\x06header\x18\x04 \x03(\x0b\x32\x39.com.spirent.service.execution.protocol.NewSession.Header\x12\x14\n\x0cparamFileUri\x18\x05 \x03(\t\x12S\n\x0brequirement\x18\x06 \x03(\x0b\x32>.com.spirent.service.execution.protocol.NewSession.Requirement\x12K\n\nproperties\x18\x07 \x01(\x0b\x32\x37.com.spirent.service.execution.protocol.PropertiesGroup\x12\x16\n\x0eresponseMapLib\x18\x08 \x01(\t\x12 \n\x11\x64isableSlcCapture\x18\t \x01(\x08:\x05\x66\x61lse\x1a%\n\x06Header\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\x1a*\n\x0bRequirement\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"\xc1\x01\n\x0cQueryProject\x12\x0f\n\x07project\x18\x01 \x02(\t\x12\x1d\n\x0fsessionProfiles\x18\x02 \x01(\x08:\x04true\x12\x18\n\ntopologies\x18\x03 \x01(\x08:\x04true\x12\x1d\n\x0eparameterFiles\x18\x04 \x01(\x08:\x05\x66\x61lse\x12\x1b\n\x0cresponseMaps\x18\x05 \x01(\x08:\x05\x66\x61lse\x12\x12\n\nspecificQC\x18\x06 \x01(\t\x12\x17\n\ttestCases\x18\x07 \x01(\x08:\x04true\"\x80\x03\n\x14QueryProjectResponse\x12\x0f\n\x07project\x18\x01 \x02(\t\x12\x17\n\x0fsessionProfiles\x18\x02 \x03(\t\x12Y\n\ntopologies\x18\x03 \x03(\x0b\x32\x45.com.spirent.service.execution.protocol.QueryProjectResponse.Topology\x12\x16\n\x0eparameterFiles\x18\x04 \x03(\t\x12\x14\n\x0cresponseMaps\x18\x05 \x03(\t\x12\x11\n\ttestCases\x18\x06 \x03(\t\x1a\xa1\x01\n\x08Topology\x12\x0c\n\x04name\x18\x01 \x02(\t\x12]\n\x07\x64\x65vices\x18\x02 \x03(\x0b\x32L.com.spirent.service.execution.protocol.QueryProjectResponse.Topology.Device\x1a(\n\x06\x44\x65vice\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x10\n\x08sessions\x18\x02 \x03(\t\"\x99\x02\n\x12NewSessionResponse\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x10\n\x08\x64uration\x18\x02 \x01(\x01\x12V\n\nstepResult\x18\x03 \x01(\x0e\x32\x39.com.spirent.service.execution.protocol.SessionStepResult:\x07SUCCESS\x12X\n\x07\x64\x65tails\x18\x04 \x01(\x0b\x32G.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails\x12\x1b\n\rallowTerminal\x18\x05 \x01(\x08:\x04true\x12\x0f\n\x07\x61gentId\x18\x06 \x01(\t\"3\n\x10SendTerminalData\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\"\xc3\x01\n\x18SendTerminalDataResponse\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\x12Y\n\x07\x63\x61pture\x18\x03 \x01(\x0b\x32H.com.spirent.service.execution.protocol.SendTerminalDataResponse.Capture\x1a+\n\x07\x43\x61pture\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\x12\x10\n\x08response\x18\x02 \x01(\t\"\x9b\x03\n\rSessionAction\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x02(\t\x12\x0f\n\x07\x63ommand\x18\x03 \x01(\t\x12\x41\n\nparameters\x18\x04 \x03(\x0b\x32-.com.spirent.service.execution.protocol.Param\x12g\n\x0fresponseMapMode\x18\x05 \x01(\x0e\x32\x45.com.spirent.service.execution.protocol.SessionAction.ResponseModeMap:\x07\x44\x45\x46\x41ULT\x12\x13\n\x0bresponseMap\x18\x06 \x01(\t\x12K\n\nproperties\x18\x07 \x01(\x0b\x32\x37.com.spirent.service.execution.protocol.PropertiesGroup\"H\n\x0fResponseModeMap\x12\x08\n\x04\x41UTO\x10\x00\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x01\x12\x08\n\x04\x46ILE\x10\x02\x12\x0b\n\x07LIBRARY\x10\x03\x12\x07\n\x03OFF\x10\x04\"\xd8\x06\n\x15SessionActionResponse\x12\x11\n\tsessionId\x18\x01 \x02(\t\x12\x11\n\tstartTime\x18\x07 \x01(\x04\x12\x10\n\x08\x64uration\x18\x02 \x01(\x01\x12V\n\nstepResult\x18\x03 \x01(\x0e\x32\x39.com.spirent.service.execution.protocol.SessionStepResult:\x07SUCCESS\x12X\n\x07\x64\x65tails\x18\x04 \x01(\x0b\x32G.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails\x12n\n\x13nestedActionDetails\x18\x05 \x01(\x0b\x32Q.com.spirent.service.execution.protocol.SessionActionResponse.SessionActionUpdate\x12o\n\x0c\x63hunkDetails\x18\x06 \x01(\x0b\x32Y.com.spirent.service.execution.protocol.SessionActionResponse.SessionResponseChunkDetails\x12\x13\n\x0b\x65xecutionId\x18\x08 \x01(\t\x1a\xd5\x01\n\x13SessionActionUpdate\x12\x45\n\x06\x61\x63tion\x18\x01 \x01(\x0b\x32\x35.com.spirent.service.execution.protocol.SessionAction\x12g\n\x06status\x18\x02 \x02(\x0e\x32W.com.spirent.service.execution.protocol.SessionActionResponse.SessionActionUpdateStatus\x12\x0e\n\x06stepId\x18\x03 \x02(\t\x1aN\n\x1bSessionResponseChunkDetails\x12\r\n\x05\x63ount\x18\x01 \x02(\x04\x12\r\n\x05index\x18\x02 \x02(\x04\x12\x11\n\tmessageId\x18\x03 \x02(\t\"7\n\x19SessionActionUpdateStatus\x12\x0b\n\x07STARTED\x10\x00\x12\r\n\tCOMPLETED\x10\x01\"\xb1\x02\n\x14SessionActionCapture\x12\x45\n\x06\x61\x63tion\x18\x01 \x02(\x0b\x32\x35.com.spirent.service.execution.protocol.SessionAction\x12O\n\x08response\x18\x02 \x02(\x0b\x32=.com.spirent.service.execution.protocol.SessionActionResponse\x12\x12\n\nsessionUri\x18\x03 \x02(\t\x12\r\n\x05token\x18\x04 \x02(\t\x12\x12\n\ndeviceName\x18\x05 \x01(\t\x12\x13\n\x0bsessionName\x18\x06 \x01(\t\x12\x0e\n\x06userId\x18\x07 \x01(\t\x12\x10\n\x08userName\x18\x08 \x01(\t\x12\x13\n\x0bsessionType\x18\t \x01(\t\"\xd9\x01\n\x11SessionTerminated\x12\x62\n\x06status\x18\x01 \x02(\x0e\x32@.com.spirent.service.execution.protocol.SessionTerminated.Status:\x10\x41GENT_TERMINATED\x12\x11\n\tsessionId\x18\x02 \x01(\t\"M\n\x06Status\x12\x14\n\x10\x41GENT_TERMINATED\x10\x01\x12\x15\n\x11\x43LIENT_DISCONNECT\x10\x02\x12\x16\n\x12SESSION_DISCONNECT\x10\x03\"6\n\x0cQuerySession\x12\x12\n\nsessionUri\x18\x01 \x02(\t\x12\x12\n\nspecificQC\x18\x02 \x01(\t\"\xec\x02\n\x14QuerySessionResponse\x12Y\n\tquickCall\x18\x02 \x03(\x0b\x32\x46.com.spirent.service.execution.protocol.QuerySessionResponse.QuickCall\x1a\xf8\x01\n\tQuickCall\x12\x0c\n\x04name\x18\x01 \x02(\t\x12]\n\x04\x61rgs\x18\x02 \x03(\x0b\x32O.com.spirent.service.execution.protocol.QuerySessionResponse.QuickCall.Argument\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0b\n\x03uri\x18\x04 \x01(\t\x1a\\\n\x08\x41rgument\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x0c\x64\x65\x66\x61ultValue\x18\x03 \x01(\t\x12\x17\n\x08required\x18\x04 \x01(\x08:\x05\x66\x61lse\"$\n\rQueryTestCase\x12\x13\n\x0btestCaseUri\x18\x01 \x02(\t\"\xca\x02\n\x15QueryTestCaseResponse\x12[\n\nprocedures\x18\x02 \x03(\x0b\x32G.com.spirent.service.execution.protocol.QueryTestCaseResponse.Procedure\x1a\xd3\x01\n\tProcedure\x12\x0c\n\x04name\x18\x01 \x02(\t\x12^\n\x04\x61rgs\x18\x02 \x03(\x0b\x32P.com.spirent.service.execution.protocol.QueryTestCaseResponse.Procedure.Argument\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1a\x43\n\x08\x41rgument\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x0c\x64\x65\x66\x61ultValue\x18\x03 \x01(\t\"\xcd\x01\n\x11TestCaseProcedure\x12\x14\n\x0cprocedureUri\x18\x01 \x02(\t\x12\x41\n\nparameters\x18\x02 \x03(\x0b\x32-.com.spirent.service.execution.protocol.Param\x12\x12\n\ntestbedUri\x18\x03 \x01(\t\x12K\n\nproperties\x18\x04 \x01(\x0b\x32\x37.com.spirent.service.execution.protocol.PropertiesGroup\"\xe4\x01\n\x19TestCaseProcedureResponse\x12\x10\n\x08\x64uration\x18\x01 \x01(\x01\x12[\n\nstepResult\x18\x02 \x01(\x0e\x32>.com.spirent.service.execution.protocol.StepMessage.StepResult:\x07SUCCESS\x12X\n\x07\x64\x65tails\x18\x03 \x01(\x0b\x32G.com.spirent.service.execution.protocol.StepMessage.StepResponseDetails\"\x18\n\x07\x45ncrypt\x12\r\n\x05value\x18\x01 \x02(\t\" \n\x0f\x45ncryptResponse\x12\r\n\x05value\x18\x01 \x02(\t*W\n\x0cReportResult\x12\x08\n\x04PASS\x10\x00\x12\x11\n\rINDETERMINATE\x10\x01\x12\x08\n\x04\x46\x41IL\x10\x02\x12\t\n\x05\x41\x42ORT\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x12\n\n\x06\x43\x41NCEL\x10\x05*S\n\x11SessionStepResult\x12\x0b\n\x07\x41\x42ORTED\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\x12\x0b\n\x07SKIPPED\x10\x02\x12\x0b\n\x07SUCCESS\x10\x03\x12\x0b\n\x07TIMEOUT\x10\x04\x42\x02H\x01')

_REPORTRESULT = DESCRIPTOR.enum_types_by_name['ReportResult']
ReportResult = enum_type_wrapper.EnumTypeWrapper(_REPORTRESULT)
_SESSIONSTEPRESULT = DESCRIPTOR.enum_types_by_name['SessionStepResult']
SessionStepResult = enum_type_wrapper.EnumTypeWrapper(_SESSIONSTEPRESULT)
PASS = 0
INDETERMINATE = 1
FAIL = 2
ABORT = 3
ERROR = 4
CANCEL = 5
ABORTED = 0
FAILED = 1
SKIPPED = 2
SUCCESS = 3
TIMEOUT = 4


_EXECUTIONMESSAGE = DESCRIPTOR.message_types_by_name['ExecutionMessage']
_AGENTSTATUSMESSAGE = DESCRIPTOR.message_types_by_name['AgentStatusMessage']
_CANCELMESSAGE = DESCRIPTOR.message_types_by_name['CancelMessage']
_EXECUTIONISSUEMESSAGE = DESCRIPTOR.message_types_by_name['ExecutionIssueMessage']
_EXTRACTEDDATAMESSAGE = DESCRIPTOR.message_types_by_name['ExtractedDataMessage']
_HEARTBEATMESSAGE = DESCRIPTOR.message_types_by_name['HeartbeatMessage']
_INVOKEACK = DESCRIPTOR.message_types_by_name['InvokeAck']
_PARAM = DESCRIPTOR.message_types_by_name['Param']
_INVOKEMESSAGE = DESCRIPTOR.message_types_by_name['InvokeMessage']
_INVOKEMESSAGE_HEADER = _INVOKEMESSAGE.nested_types_by_name['Header']
_INVOKEMESSAGE_RESERVATION = _INVOKEMESSAGE.nested_types_by_name['Reservation']
_INITMESSAGE = DESCRIPTOR.message_types_by_name['InitMessage']
_INITMESSAGE_CAPABILITY = _INITMESSAGE.nested_types_by_name['Capability']
_INITMESSAGE_RESTRICTION = _INITMESSAGE.nested_types_by_name['Restriction']
_INITMESSAGEACK = DESCRIPTOR.message_types_by_name['InitMessageAck']
_INITMESSAGEACK_CONFIGSETTINGS = _INITMESSAGEACK.nested_types_by_name['ConfigSettings']
_INVOKERESPONSE = DESCRIPTOR.message_types_by_name['InvokeResponse']
_INVOKERESPONSE_ERROR = _INVOKERESPONSE.nested_types_by_name['Error']
_TERMINATEMESSAGE = DESCRIPTOR.message_types_by_name['TerminateMessage']
_TOKENMESSAGE = DESCRIPTOR.message_types_by_name['TokenMessage']
_REPORTMESSAGE = DESCRIPTOR.message_types_by_name['ReportMessage']
_REPORTMESSAGE_KEYVALUE = _REPORTMESSAGE.nested_types_by_name['KeyValue']
_STEPMESSAGE = DESCRIPTOR.message_types_by_name['StepMessage']
_STEPMESSAGE_ACTION = _STEPMESSAGE.nested_types_by_name['Action']
_STEPMESSAGE_STEPRESPONSEDETAILS = _STEPMESSAGE.nested_types_by_name['StepResponseDetails']
_STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS = _STEPMESSAGE_STEPRESPONSEDETAILS.nested_types_by_name['Alias']
_STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS_ARGUMENT = _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS.nested_types_by_name['Argument']
_STEPMESSAGE_STEPRESPONSEDETAILS_ANALYSISRULE = _STEPMESSAGE_STEPRESPONSEDETAILS.nested_types_by_name['AnalysisRule']
_STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA = _STEPMESSAGE_STEPRESPONSEDETAILS.nested_types_by_name['StructuredData']
_STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA_ITEM = _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA.nested_types_by_name['Item']
_CLIENTSESSIONCONTROLMESSAGE = DESCRIPTOR.message_types_by_name['ClientSessionControlMessage']
_INIT = DESCRIPTOR.message_types_by_name['Init']
_INITRESPONSE = DESCRIPTOR.message_types_by_name['InitResponse']
_LISTPROJECTS = DESCRIPTOR.message_types_by_name['ListProjects']
_LISTPROJECTSRESPONSE = DESCRIPTOR.message_types_by_name['ListProjectsResponse']
_PROPERTIESGROUP = DESCRIPTOR.message_types_by_name['PropertiesGroup']
_PROPERTIESGROUP_PROPERTY = _PROPERTIESGROUP.nested_types_by_name['Property']
_NEWSESSION = DESCRIPTOR.message_types_by_name['NewSession']
_NEWSESSION_HEADER = _NEWSESSION.nested_types_by_name['Header']
_NEWSESSION_REQUIREMENT = _NEWSESSION.nested_types_by_name['Requirement']
_QUERYPROJECT = DESCRIPTOR.message_types_by_name['QueryProject']
_QUERYPROJECTRESPONSE = DESCRIPTOR.message_types_by_name['QueryProjectResponse']
_QUERYPROJECTRESPONSE_TOPOLOGY = _QUERYPROJECTRESPONSE.nested_types_by_name['Topology']
_QUERYPROJECTRESPONSE_TOPOLOGY_DEVICE = _QUERYPROJECTRESPONSE_TOPOLOGY.nested_types_by_name['Device']
_NEWSESSIONRESPONSE = DESCRIPTOR.message_types_by_name['NewSessionResponse']
_SENDTERMINALDATA = DESCRIPTOR.message_types_by_name['SendTerminalData']
_SENDTERMINALDATARESPONSE = DESCRIPTOR.message_types_by_name['SendTerminalDataResponse']
_SENDTERMINALDATARESPONSE_CAPTURE = _SENDTERMINALDATARESPONSE.nested_types_by_name['Capture']
_SESSIONACTION = DESCRIPTOR.message_types_by_name['SessionAction']
_SESSIONACTIONRESPONSE = DESCRIPTOR.message_types_by_name['SessionActionResponse']
_SESSIONACTIONRESPONSE_SESSIONACTIONUPDATE = _SESSIONACTIONRESPONSE.nested_types_by_name['SessionActionUpdate']
_SESSIONACTIONRESPONSE_SESSIONRESPONSECHUNKDETAILS = _SESSIONACTIONRESPONSE.nested_types_by_name['SessionResponseChunkDetails']
_SESSIONACTIONCAPTURE = DESCRIPTOR.message_types_by_name['SessionActionCapture']
_SESSIONTERMINATED = DESCRIPTOR.message_types_by_name['SessionTerminated']
_QUERYSESSION = DESCRIPTOR.message_types_by_name['QuerySession']
_QUERYSESSIONRESPONSE = DESCRIPTOR.message_types_by_name['QuerySessionResponse']
_QUERYSESSIONRESPONSE_QUICKCALL = _QUERYSESSIONRESPONSE.nested_types_by_name['QuickCall']
_QUERYSESSIONRESPONSE_QUICKCALL_ARGUMENT = _QUERYSESSIONRESPONSE_QUICKCALL.nested_types_by_name['Argument']
_QUERYTESTCASE = DESCRIPTOR.message_types_by_name['QueryTestCase']
_QUERYTESTCASERESPONSE = DESCRIPTOR.message_types_by_name['QueryTestCaseResponse']
_QUERYTESTCASERESPONSE_PROCEDURE = _QUERYTESTCASERESPONSE.nested_types_by_name['Procedure']
_QUERYTESTCASERESPONSE_PROCEDURE_ARGUMENT = _QUERYTESTCASERESPONSE_PROCEDURE.nested_types_by_name['Argument']
_TESTCASEPROCEDURE = DESCRIPTOR.message_types_by_name['TestCaseProcedure']
_TESTCASEPROCEDURERESPONSE = DESCRIPTOR.message_types_by_name['TestCaseProcedureResponse']
_ENCRYPT = DESCRIPTOR.message_types_by_name['Encrypt']
_ENCRYPTRESPONSE = DESCRIPTOR.message_types_by_name['EncryptResponse']
_EXECUTIONMESSAGE_TYPE = _EXECUTIONMESSAGE.enum_types_by_name['Type']
_EXECUTIONISSUEMESSAGE_SEVERITY = _EXECUTIONISSUEMESSAGE.enum_types_by_name['Severity']
_INVOKEMESSAGE_DETAILLEVEL = _INVOKEMESSAGE.enum_types_by_name['DetailLevel']
_REPORTMESSAGE_EXECUTIONSTATE = _REPORTMESSAGE.enum_types_by_name['ExecutionState']
_REPORTMESSAGE_DETAILLEVEL = _REPORTMESSAGE.enum_types_by_name['DetailLevel']
_STEPMESSAGE_STEPRESULT = _STEPMESSAGE.enum_types_by_name['StepResult']
_CLIENTSESSIONCONTROLMESSAGE_TYPE = _CLIENTSESSIONCONTROLMESSAGE.enum_types_by_name['Type']
_SESSIONACTION_RESPONSEMODEMAP = _SESSIONACTION.enum_types_by_name['ResponseModeMap']
_SESSIONACTIONRESPONSE_SESSIONACTIONUPDATESTATUS = _SESSIONACTIONRESPONSE.enum_types_by_name['SessionActionUpdateStatus']
_SESSIONTERMINATED_STATUS = _SESSIONTERMINATED.enum_types_by_name['Status']
ExecutionMessage = _reflection.GeneratedProtocolMessageType('ExecutionMessage', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTIONMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ExecutionMessage)
  })
_sym_db.RegisterMessage(ExecutionMessage)

AgentStatusMessage = _reflection.GeneratedProtocolMessageType('AgentStatusMessage', (_message.Message,), {
  'DESCRIPTOR' : _AGENTSTATUSMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.AgentStatusMessage)
  })
_sym_db.RegisterMessage(AgentStatusMessage)

CancelMessage = _reflection.GeneratedProtocolMessageType('CancelMessage', (_message.Message,), {
  'DESCRIPTOR' : _CANCELMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.CancelMessage)
  })
_sym_db.RegisterMessage(CancelMessage)

ExecutionIssueMessage = _reflection.GeneratedProtocolMessageType('ExecutionIssueMessage', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTIONISSUEMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ExecutionIssueMessage)
  })
_sym_db.RegisterMessage(ExecutionIssueMessage)

ExtractedDataMessage = _reflection.GeneratedProtocolMessageType('ExtractedDataMessage', (_message.Message,), {
  'DESCRIPTOR' : _EXTRACTEDDATAMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ExtractedDataMessage)
  })
_sym_db.RegisterMessage(ExtractedDataMessage)

HeartbeatMessage = _reflection.GeneratedProtocolMessageType('HeartbeatMessage', (_message.Message,), {
  'DESCRIPTOR' : _HEARTBEATMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.HeartbeatMessage)
  })
_sym_db.RegisterMessage(HeartbeatMessage)

InvokeAck = _reflection.GeneratedProtocolMessageType('InvokeAck', (_message.Message,), {
  'DESCRIPTOR' : _INVOKEACK,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InvokeAck)
  })
_sym_db.RegisterMessage(InvokeAck)

Param = _reflection.GeneratedProtocolMessageType('Param', (_message.Message,), {
  'DESCRIPTOR' : _PARAM,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.Param)
  })
_sym_db.RegisterMessage(Param)

InvokeMessage = _reflection.GeneratedProtocolMessageType('InvokeMessage', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _INVOKEMESSAGE_HEADER,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InvokeMessage.Header)
    })
  ,

  'Reservation' : _reflection.GeneratedProtocolMessageType('Reservation', (_message.Message,), {
    'DESCRIPTOR' : _INVOKEMESSAGE_RESERVATION,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InvokeMessage.Reservation)
    })
  ,
  'DESCRIPTOR' : _INVOKEMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InvokeMessage)
  })
_sym_db.RegisterMessage(InvokeMessage)
_sym_db.RegisterMessage(InvokeMessage.Header)
_sym_db.RegisterMessage(InvokeMessage.Reservation)

InitMessage = _reflection.GeneratedProtocolMessageType('InitMessage', (_message.Message,), {

  'Capability' : _reflection.GeneratedProtocolMessageType('Capability', (_message.Message,), {
    'DESCRIPTOR' : _INITMESSAGE_CAPABILITY,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InitMessage.Capability)
    })
  ,

  'Restriction' : _reflection.GeneratedProtocolMessageType('Restriction', (_message.Message,), {
    'DESCRIPTOR' : _INITMESSAGE_RESTRICTION,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InitMessage.Restriction)
    })
  ,
  'DESCRIPTOR' : _INITMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InitMessage)
  })
_sym_db.RegisterMessage(InitMessage)
_sym_db.RegisterMessage(InitMessage.Capability)
_sym_db.RegisterMessage(InitMessage.Restriction)

InitMessageAck = _reflection.GeneratedProtocolMessageType('InitMessageAck', (_message.Message,), {

  'ConfigSettings' : _reflection.GeneratedProtocolMessageType('ConfigSettings', (_message.Message,), {
    'DESCRIPTOR' : _INITMESSAGEACK_CONFIGSETTINGS,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InitMessageAck.ConfigSettings)
    })
  ,
  'DESCRIPTOR' : _INITMESSAGEACK,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InitMessageAck)
  })
_sym_db.RegisterMessage(InitMessageAck)
_sym_db.RegisterMessage(InitMessageAck.ConfigSettings)

InvokeResponse = _reflection.GeneratedProtocolMessageType('InvokeResponse', (_message.Message,), {

  'Error' : _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), {
    'DESCRIPTOR' : _INVOKERESPONSE_ERROR,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InvokeResponse.Error)
    })
  ,
  'DESCRIPTOR' : _INVOKERESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InvokeResponse)
  })
_sym_db.RegisterMessage(InvokeResponse)
_sym_db.RegisterMessage(InvokeResponse.Error)

TerminateMessage = _reflection.GeneratedProtocolMessageType('TerminateMessage', (_message.Message,), {
  'DESCRIPTOR' : _TERMINATEMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.TerminateMessage)
  })
_sym_db.RegisterMessage(TerminateMessage)

TokenMessage = _reflection.GeneratedProtocolMessageType('TokenMessage', (_message.Message,), {
  'DESCRIPTOR' : _TOKENMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.TokenMessage)
  })
_sym_db.RegisterMessage(TokenMessage)

ReportMessage = _reflection.GeneratedProtocolMessageType('ReportMessage', (_message.Message,), {

  'KeyValue' : _reflection.GeneratedProtocolMessageType('KeyValue', (_message.Message,), {
    'DESCRIPTOR' : _REPORTMESSAGE_KEYVALUE,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ReportMessage.KeyValue)
    })
  ,
  'DESCRIPTOR' : _REPORTMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ReportMessage)
  })
_sym_db.RegisterMessage(ReportMessage)
_sym_db.RegisterMessage(ReportMessage.KeyValue)

StepMessage = _reflection.GeneratedProtocolMessageType('StepMessage', (_message.Message,), {

  'Action' : _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
    'DESCRIPTOR' : _STEPMESSAGE_ACTION,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.Action)
    })
  ,

  'StepResponseDetails' : _reflection.GeneratedProtocolMessageType('StepResponseDetails', (_message.Message,), {

    'Alias' : _reflection.GeneratedProtocolMessageType('Alias', (_message.Message,), {

      'Argument' : _reflection.GeneratedProtocolMessageType('Argument', (_message.Message,), {
        'DESCRIPTOR' : _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS_ARGUMENT,
        '__module__' : 'ExecutionProtocol_pb2'
        # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.Alias.Argument)
        })
      ,
      'DESCRIPTOR' : _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS,
      '__module__' : 'ExecutionProtocol_pb2'
      # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.Alias)
      })
    ,

    'AnalysisRule' : _reflection.GeneratedProtocolMessageType('AnalysisRule', (_message.Message,), {
      'DESCRIPTOR' : _STEPMESSAGE_STEPRESPONSEDETAILS_ANALYSISRULE,
      '__module__' : 'ExecutionProtocol_pb2'
      # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.AnalysisRule)
      })
    ,

    'StructuredData' : _reflection.GeneratedProtocolMessageType('StructuredData', (_message.Message,), {

      'Item' : _reflection.GeneratedProtocolMessageType('Item', (_message.Message,), {
        'DESCRIPTOR' : _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA_ITEM,
        '__module__' : 'ExecutionProtocol_pb2'
        # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.StructuredData.Item)
        })
      ,
      'DESCRIPTOR' : _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA,
      '__module__' : 'ExecutionProtocol_pb2'
      # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.StepResponseDetails.StructuredData)
      })
    ,
    'DESCRIPTOR' : _STEPMESSAGE_STEPRESPONSEDETAILS,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage.StepResponseDetails)
    })
  ,
  'DESCRIPTOR' : _STEPMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.StepMessage)
  })
_sym_db.RegisterMessage(StepMessage)
_sym_db.RegisterMessage(StepMessage.Action)
_sym_db.RegisterMessage(StepMessage.StepResponseDetails)
_sym_db.RegisterMessage(StepMessage.StepResponseDetails.Alias)
_sym_db.RegisterMessage(StepMessage.StepResponseDetails.Alias.Argument)
_sym_db.RegisterMessage(StepMessage.StepResponseDetails.AnalysisRule)
_sym_db.RegisterMessage(StepMessage.StepResponseDetails.StructuredData)
_sym_db.RegisterMessage(StepMessage.StepResponseDetails.StructuredData.Item)

ClientSessionControlMessage = _reflection.GeneratedProtocolMessageType('ClientSessionControlMessage', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTSESSIONCONTROLMESSAGE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ClientSessionControlMessage)
  })
_sym_db.RegisterMessage(ClientSessionControlMessage)

Init = _reflection.GeneratedProtocolMessageType('Init', (_message.Message,), {
  'DESCRIPTOR' : _INIT,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.Init)
  })
_sym_db.RegisterMessage(Init)

InitResponse = _reflection.GeneratedProtocolMessageType('InitResponse', (_message.Message,), {
  'DESCRIPTOR' : _INITRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.InitResponse)
  })
_sym_db.RegisterMessage(InitResponse)

ListProjects = _reflection.GeneratedProtocolMessageType('ListProjects', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROJECTS,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ListProjects)
  })
_sym_db.RegisterMessage(ListProjects)

ListProjectsResponse = _reflection.GeneratedProtocolMessageType('ListProjectsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROJECTSRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.ListProjectsResponse)
  })
_sym_db.RegisterMessage(ListProjectsResponse)

PropertiesGroup = _reflection.GeneratedProtocolMessageType('PropertiesGroup', (_message.Message,), {

  'Property' : _reflection.GeneratedProtocolMessageType('Property', (_message.Message,), {
    'DESCRIPTOR' : _PROPERTIESGROUP_PROPERTY,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.PropertiesGroup.Property)
    })
  ,
  'DESCRIPTOR' : _PROPERTIESGROUP,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.PropertiesGroup)
  })
_sym_db.RegisterMessage(PropertiesGroup)
_sym_db.RegisterMessage(PropertiesGroup.Property)

NewSession = _reflection.GeneratedProtocolMessageType('NewSession', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _NEWSESSION_HEADER,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.NewSession.Header)
    })
  ,

  'Requirement' : _reflection.GeneratedProtocolMessageType('Requirement', (_message.Message,), {
    'DESCRIPTOR' : _NEWSESSION_REQUIREMENT,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.NewSession.Requirement)
    })
  ,
  'DESCRIPTOR' : _NEWSESSION,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.NewSession)
  })
_sym_db.RegisterMessage(NewSession)
_sym_db.RegisterMessage(NewSession.Header)
_sym_db.RegisterMessage(NewSession.Requirement)

QueryProject = _reflection.GeneratedProtocolMessageType('QueryProject', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPROJECT,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryProject)
  })
_sym_db.RegisterMessage(QueryProject)

QueryProjectResponse = _reflection.GeneratedProtocolMessageType('QueryProjectResponse', (_message.Message,), {

  'Topology' : _reflection.GeneratedProtocolMessageType('Topology', (_message.Message,), {

    'Device' : _reflection.GeneratedProtocolMessageType('Device', (_message.Message,), {
      'DESCRIPTOR' : _QUERYPROJECTRESPONSE_TOPOLOGY_DEVICE,
      '__module__' : 'ExecutionProtocol_pb2'
      # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryProjectResponse.Topology.Device)
      })
    ,
    'DESCRIPTOR' : _QUERYPROJECTRESPONSE_TOPOLOGY,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryProjectResponse.Topology)
    })
  ,
  'DESCRIPTOR' : _QUERYPROJECTRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryProjectResponse)
  })
_sym_db.RegisterMessage(QueryProjectResponse)
_sym_db.RegisterMessage(QueryProjectResponse.Topology)
_sym_db.RegisterMessage(QueryProjectResponse.Topology.Device)

NewSessionResponse = _reflection.GeneratedProtocolMessageType('NewSessionResponse', (_message.Message,), {
  'DESCRIPTOR' : _NEWSESSIONRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.NewSessionResponse)
  })
_sym_db.RegisterMessage(NewSessionResponse)

SendTerminalData = _reflection.GeneratedProtocolMessageType('SendTerminalData', (_message.Message,), {
  'DESCRIPTOR' : _SENDTERMINALDATA,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SendTerminalData)
  })
_sym_db.RegisterMessage(SendTerminalData)

SendTerminalDataResponse = _reflection.GeneratedProtocolMessageType('SendTerminalDataResponse', (_message.Message,), {

  'Capture' : _reflection.GeneratedProtocolMessageType('Capture', (_message.Message,), {
    'DESCRIPTOR' : _SENDTERMINALDATARESPONSE_CAPTURE,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SendTerminalDataResponse.Capture)
    })
  ,
  'DESCRIPTOR' : _SENDTERMINALDATARESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SendTerminalDataResponse)
  })
_sym_db.RegisterMessage(SendTerminalDataResponse)
_sym_db.RegisterMessage(SendTerminalDataResponse.Capture)

SessionAction = _reflection.GeneratedProtocolMessageType('SessionAction', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONACTION,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SessionAction)
  })
_sym_db.RegisterMessage(SessionAction)

SessionActionResponse = _reflection.GeneratedProtocolMessageType('SessionActionResponse', (_message.Message,), {

  'SessionActionUpdate' : _reflection.GeneratedProtocolMessageType('SessionActionUpdate', (_message.Message,), {
    'DESCRIPTOR' : _SESSIONACTIONRESPONSE_SESSIONACTIONUPDATE,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SessionActionResponse.SessionActionUpdate)
    })
  ,

  'SessionResponseChunkDetails' : _reflection.GeneratedProtocolMessageType('SessionResponseChunkDetails', (_message.Message,), {
    'DESCRIPTOR' : _SESSIONACTIONRESPONSE_SESSIONRESPONSECHUNKDETAILS,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SessionActionResponse.SessionResponseChunkDetails)
    })
  ,
  'DESCRIPTOR' : _SESSIONACTIONRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SessionActionResponse)
  })
_sym_db.RegisterMessage(SessionActionResponse)
_sym_db.RegisterMessage(SessionActionResponse.SessionActionUpdate)
_sym_db.RegisterMessage(SessionActionResponse.SessionResponseChunkDetails)

SessionActionCapture = _reflection.GeneratedProtocolMessageType('SessionActionCapture', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONACTIONCAPTURE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SessionActionCapture)
  })
_sym_db.RegisterMessage(SessionActionCapture)

SessionTerminated = _reflection.GeneratedProtocolMessageType('SessionTerminated', (_message.Message,), {
  'DESCRIPTOR' : _SESSIONTERMINATED,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.SessionTerminated)
  })
_sym_db.RegisterMessage(SessionTerminated)

QuerySession = _reflection.GeneratedProtocolMessageType('QuerySession', (_message.Message,), {
  'DESCRIPTOR' : _QUERYSESSION,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QuerySession)
  })
_sym_db.RegisterMessage(QuerySession)

QuerySessionResponse = _reflection.GeneratedProtocolMessageType('QuerySessionResponse', (_message.Message,), {

  'QuickCall' : _reflection.GeneratedProtocolMessageType('QuickCall', (_message.Message,), {

    'Argument' : _reflection.GeneratedProtocolMessageType('Argument', (_message.Message,), {
      'DESCRIPTOR' : _QUERYSESSIONRESPONSE_QUICKCALL_ARGUMENT,
      '__module__' : 'ExecutionProtocol_pb2'
      # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QuerySessionResponse.QuickCall.Argument)
      })
    ,
    'DESCRIPTOR' : _QUERYSESSIONRESPONSE_QUICKCALL,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QuerySessionResponse.QuickCall)
    })
  ,
  'DESCRIPTOR' : _QUERYSESSIONRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QuerySessionResponse)
  })
_sym_db.RegisterMessage(QuerySessionResponse)
_sym_db.RegisterMessage(QuerySessionResponse.QuickCall)
_sym_db.RegisterMessage(QuerySessionResponse.QuickCall.Argument)

QueryTestCase = _reflection.GeneratedProtocolMessageType('QueryTestCase', (_message.Message,), {
  'DESCRIPTOR' : _QUERYTESTCASE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryTestCase)
  })
_sym_db.RegisterMessage(QueryTestCase)

QueryTestCaseResponse = _reflection.GeneratedProtocolMessageType('QueryTestCaseResponse', (_message.Message,), {

  'Procedure' : _reflection.GeneratedProtocolMessageType('Procedure', (_message.Message,), {

    'Argument' : _reflection.GeneratedProtocolMessageType('Argument', (_message.Message,), {
      'DESCRIPTOR' : _QUERYTESTCASERESPONSE_PROCEDURE_ARGUMENT,
      '__module__' : 'ExecutionProtocol_pb2'
      # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryTestCaseResponse.Procedure.Argument)
      })
    ,
    'DESCRIPTOR' : _QUERYTESTCASERESPONSE_PROCEDURE,
    '__module__' : 'ExecutionProtocol_pb2'
    # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryTestCaseResponse.Procedure)
    })
  ,
  'DESCRIPTOR' : _QUERYTESTCASERESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.QueryTestCaseResponse)
  })
_sym_db.RegisterMessage(QueryTestCaseResponse)
_sym_db.RegisterMessage(QueryTestCaseResponse.Procedure)
_sym_db.RegisterMessage(QueryTestCaseResponse.Procedure.Argument)

TestCaseProcedure = _reflection.GeneratedProtocolMessageType('TestCaseProcedure', (_message.Message,), {
  'DESCRIPTOR' : _TESTCASEPROCEDURE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.TestCaseProcedure)
  })
_sym_db.RegisterMessage(TestCaseProcedure)

TestCaseProcedureResponse = _reflection.GeneratedProtocolMessageType('TestCaseProcedureResponse', (_message.Message,), {
  'DESCRIPTOR' : _TESTCASEPROCEDURERESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.TestCaseProcedureResponse)
  })
_sym_db.RegisterMessage(TestCaseProcedureResponse)

Encrypt = _reflection.GeneratedProtocolMessageType('Encrypt', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPT,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.Encrypt)
  })
_sym_db.RegisterMessage(Encrypt)

EncryptResponse = _reflection.GeneratedProtocolMessageType('EncryptResponse', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPTRESPONSE,
  '__module__' : 'ExecutionProtocol_pb2'
  # @@protoc_insertion_point(class_scope:com.spirent.service.execution.protocol.EncryptResponse)
  })
_sym_db.RegisterMessage(EncryptResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'H\001'
  _REPORTRESULT._serialized_start=18332
  _REPORTRESULT._serialized_end=18419
  _SESSIONSTEPRESULT._serialized_start=18421
  _SESSIONSTEPRESULT._serialized_end=18504
  _EXECUTIONMESSAGE._serialized_start=68
  _EXECUTIONMESSAGE._serialized_end=3843
  _EXECUTIONMESSAGE_TYPE._serialized_start=3023
  _EXECUTIONMESSAGE_TYPE._serialized_end=3843
  _AGENTSTATUSMESSAGE._serialized_start=3845
  _AGENTSTATUSMESSAGE._serialized_end=3970
  _CANCELMESSAGE._serialized_start=3972
  _CANCELMESSAGE._serialized_end=4006
  _EXECUTIONISSUEMESSAGE._serialized_start=4009
  _EXECUTIONISSUEMESSAGE._serialized_end=4551
  _EXECUTIONISSUEMESSAGE_SEVERITY._serialized_start=4490
  _EXECUTIONISSUEMESSAGE_SEVERITY._serialized_end=4551
  _EXTRACTEDDATAMESSAGE._serialized_start=4554
  _EXTRACTEDDATAMESSAGE._serialized_end=4789
  _HEARTBEATMESSAGE._serialized_start=4791
  _HEARTBEATMESSAGE._serialized_end=4889
  _INVOKEACK._serialized_start=4891
  _INVOKEACK._serialized_end=4940
  _PARAM._serialized_start=4943
  _PARAM._serialized_end=5104
  _INVOKEMESSAGE._serialized_start=5107
  _INVOKEMESSAGE._serialized_end=5905
  _INVOKEMESSAGE_HEADER._serialized_start=5640
  _INVOKEMESSAGE_HEADER._serialized_end=5677
  _INVOKEMESSAGE_RESERVATION._serialized_start=5679
  _INVOKEMESSAGE_RESERVATION._serialized_end=5747
  _INVOKEMESSAGE_DETAILLEVEL._serialized_start=5750
  _INVOKEMESSAGE_DETAILLEVEL._serialized_end=5905
  _INITMESSAGE._serialized_start=5908
  _INITMESSAGE._serialized_end=6283
  _INITMESSAGE_CAPABILITY._serialized_start=6198
  _INITMESSAGE_CAPABILITY._serialized_end=6239
  _INITMESSAGE_RESTRICTION._serialized_start=6241
  _INITMESSAGE_RESTRICTION._serialized_end=6283
  _INITMESSAGEACK._serialized_start=6286
  _INITMESSAGEACK._serialized_end=6479
  _INITMESSAGEACK_CONFIGSETTINGS._serialized_start=6434
  _INITMESSAGEACK_CONFIGSETTINGS._serialized_end=6479
  _INVOKERESPONSE._serialized_start=6482
  _INVOKERESPONSE._serialized_end=6774
  _INVOKERESPONSE_ERROR._serialized_start=6736
  _INVOKERESPONSE_ERROR._serialized_end=6774
  _TERMINATEMESSAGE._serialized_start=6776
  _TERMINATEMESSAGE._serialized_end=6809
  _TOKENMESSAGE._serialized_start=6811
  _TOKENMESSAGE._serialized_end=6840
  _REPORTMESSAGE._serialized_start=6843
  _REPORTMESSAGE._serialized_end=8861
  _REPORTMESSAGE_KEYVALUE._serialized_start=8524
  _REPORTMESSAGE_KEYVALUE._serialized_end=8563
  _REPORTMESSAGE_EXECUTIONSTATE._serialized_start=8566
  _REPORTMESSAGE_EXECUTIONSTATE._serialized_end=8703
  _REPORTMESSAGE_DETAILLEVEL._serialized_start=5750
  _REPORTMESSAGE_DETAILLEVEL._serialized_end=5905
  _STEPMESSAGE._serialized_start=8864
  _STEPMESSAGE._serialized_end=10864
  _STEPMESSAGE_ACTION._serialized_start=9401
  _STEPMESSAGE_ACTION._serialized_end=9523
  _STEPMESSAGE_STEPRESPONSEDETAILS._serialized_start=9526
  _STEPMESSAGE_STEPRESPONSEDETAILS._serialized_end=10786
  _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS._serialized_start=9994
  _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS._serialized_end=10238
  _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS_ARGUMENT._serialized_start=10172
  _STEPMESSAGE_STEPRESPONSEDETAILS_ALIAS_ARGUMENT._serialized_end=10238
  _STEPMESSAGE_STEPRESPONSEDETAILS_ANALYSISRULE._serialized_start=10241
  _STEPMESSAGE_STEPRESPONSEDETAILS_ANALYSISRULE._serialized_end=10409
  _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA._serialized_start=10412
  _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA._serialized_end=10786
  _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA_ITEM._serialized_start=10576
  _STEPMESSAGE_STEPRESPONSEDETAILS_STRUCTUREDDATA_ITEM._serialized_end=10786
  _STEPMESSAGE_STEPRESULT._serialized_start=10788
  _STEPMESSAGE_STEPRESULT._serialized_end=10864
  _CLIENTSESSIONCONTROLMESSAGE._serialized_start=10867
  _CLIENTSESSIONCONTROLMESSAGE._serialized_end=13249
  _CLIENTSESSIONCONTROLMESSAGE_TYPE._serialized_start=12730
  _CLIENTSESSIONCONTROLMESSAGE_TYPE._serialized_end=13249
  _INIT._serialized_start=13251
  _INIT._serialized_end=13289
  _INITRESPONSE._serialized_start=13291
  _INITRESPONSE._serialized_end=13347
  _LISTPROJECTS._serialized_start=13349
  _LISTPROJECTS._serialized_end=13363
  _LISTPROJECTSRESPONSE._serialized_start=13365
  _LISTPROJECTSRESPONSE._serialized_end=13405
  _PROPERTIESGROUP._serialized_start=13408
  _PROPERTIESGROUP._serialized_end=13641
  _PROPERTIESGROUP_PROPERTY._serialized_start=13602
  _PROPERTIESGROUP_PROPERTY._serialized_end=13641
  _NEWSESSION._serialized_start=13644
  _NEWSESSION._serialized_end=14118
  _NEWSESSION_HEADER._serialized_start=5640
  _NEWSESSION_HEADER._serialized_end=5677
  _NEWSESSION_REQUIREMENT._serialized_start=14076
  _NEWSESSION_REQUIREMENT._serialized_end=14118
  _QUERYPROJECT._serialized_start=14121
  _QUERYPROJECT._serialized_end=14314
  _QUERYPROJECTRESPONSE._serialized_start=14317
  _QUERYPROJECTRESPONSE._serialized_end=14701
  _QUERYPROJECTRESPONSE_TOPOLOGY._serialized_start=14540
  _QUERYPROJECTRESPONSE_TOPOLOGY._serialized_end=14701
  _QUERYPROJECTRESPONSE_TOPOLOGY_DEVICE._serialized_start=14661
  _QUERYPROJECTRESPONSE_TOPOLOGY_DEVICE._serialized_end=14701
  _NEWSESSIONRESPONSE._serialized_start=14704
  _NEWSESSIONRESPONSE._serialized_end=14985
  _SENDTERMINALDATA._serialized_start=14987
  _SENDTERMINALDATA._serialized_end=15038
  _SENDTERMINALDATARESPONSE._serialized_start=15041
  _SENDTERMINALDATARESPONSE._serialized_end=15236
  _SENDTERMINALDATARESPONSE_CAPTURE._serialized_start=15193
  _SENDTERMINALDATARESPONSE_CAPTURE._serialized_end=15236
  _SESSIONACTION._serialized_start=15239
  _SESSIONACTION._serialized_end=15650
  _SESSIONACTION_RESPONSEMODEMAP._serialized_start=15578
  _SESSIONACTION_RESPONSEMODEMAP._serialized_end=15650
  _SESSIONACTIONRESPONSE._serialized_start=15653
  _SESSIONACTIONRESPONSE._serialized_end=16509
  _SESSIONACTIONRESPONSE_SESSIONACTIONUPDATE._serialized_start=16159
  _SESSIONACTIONRESPONSE_SESSIONACTIONUPDATE._serialized_end=16372
  _SESSIONACTIONRESPONSE_SESSIONRESPONSECHUNKDETAILS._serialized_start=16374
  _SESSIONACTIONRESPONSE_SESSIONRESPONSECHUNKDETAILS._serialized_end=16452
  _SESSIONACTIONRESPONSE_SESSIONACTIONUPDATESTATUS._serialized_start=16454
  _SESSIONACTIONRESPONSE_SESSIONACTIONUPDATESTATUS._serialized_end=16509
  _SESSIONACTIONCAPTURE._serialized_start=16512
  _SESSIONACTIONCAPTURE._serialized_end=16817
  _SESSIONTERMINATED._serialized_start=16820
  _SESSIONTERMINATED._serialized_end=17037
  _SESSIONTERMINATED_STATUS._serialized_start=16960
  _SESSIONTERMINATED_STATUS._serialized_end=17037
  _QUERYSESSION._serialized_start=17039
  _QUERYSESSION._serialized_end=17093
  _QUERYSESSIONRESPONSE._serialized_start=17096
  _QUERYSESSIONRESPONSE._serialized_end=17460
  _QUERYSESSIONRESPONSE_QUICKCALL._serialized_start=17212
  _QUERYSESSIONRESPONSE_QUICKCALL._serialized_end=17460
  _QUERYSESSIONRESPONSE_QUICKCALL_ARGUMENT._serialized_start=17368
  _QUERYSESSIONRESPONSE_QUICKCALL_ARGUMENT._serialized_end=17460
  _QUERYTESTCASE._serialized_start=17462
  _QUERYTESTCASE._serialized_end=17498
  _QUERYTESTCASERESPONSE._serialized_start=17501
  _QUERYTESTCASERESPONSE._serialized_end=17831
  _QUERYTESTCASERESPONSE_PROCEDURE._serialized_start=17620
  _QUERYTESTCASERESPONSE_PROCEDURE._serialized_end=17831
  _QUERYTESTCASERESPONSE_PROCEDURE_ARGUMENT._serialized_start=17368
  _QUERYTESTCASERESPONSE_PROCEDURE_ARGUMENT._serialized_end=17435
  _TESTCASEPROCEDURE._serialized_start=17834
  _TESTCASEPROCEDURE._serialized_end=18039
  _TESTCASEPROCEDURERESPONSE._serialized_start=18042
  _TESTCASEPROCEDURERESPONSE._serialized_end=18270
  _ENCRYPT._serialized_start=18272
  _ENCRYPT._serialized_end=18296
  _ENCRYPTRESPONSE._serialized_start=18298
  _ENCRYPTRESPONSE._serialized_end=18330
# @@protoc_insertion_point(module_scope)



# ****************       BEGIN-STANDARD-COPYRIGHT      ***************
#
# Copyright (c) 2017, Spirent Communications.
#
# All rights reserved. Proprietary and confidential information of Spirent Communications.
#
#  ***************        END-STANDARD-COPYRIGHT       ***************

"""Session profile class and necessary tools."""

import six

from .identity import UriIdentity
from .internal.protocol import Type, create_action_message
from .internal.response_storage import ResponseStorage
from .resources import ParameterFile, ResponseMapFile
from .session_response import SessionActionResponse, response_step_compare
from .exception import SLCError
from functools import cmp_to_key

_KNOWN_INVOKE_ARGS = ['response_map', 'parameters', 'command', 'properties']


def _to_invoke_args(kwargs):
    """Prepares the given arguments dict for invoking as action arguments.

    Arguments:
    kwargs -- dict or arguments

    Returns a new dict, ready to be passed as arguments to invoke_command().
    """

    ret = {}
    params = {}

    for key, value in kwargs.items():
        if key in _KNOWN_INVOKE_ARGS:
            ret[key] = value
        else:
            params[key] = value

    if params:
        ret['parameters'] = params
    return ret


def _to_list(arg):
    """If arg is a list, returns it. If it's None, returns None. Otherwise, returns a one-element list with arg."""

    if arg is None:
        return None

    if isinstance(arg, list):
        return arg

    return [arg, ]


def _to_requirements(agent_requirements):
    """Converts the given argument into a list of Type.Requirement.

    Arguments:
    agent_requirements -- dict of the format {'requirement_name': 'requirement_value'}. Can be None.

    Returns a list of Type.Requirement, or None.
    """

    if not agent_requirements:
        return None

    ret = []
    for key, value in agent_requirements.items():
        req = Type.Requirement()
        req.name = key
        req.value = value
        ret.append(req)

    return ret


def _to_property_group(properties):
    """Converts the given properties dict into Type.PropertiesGroup.

    Arguments:
        properties -- a dictionary with keys being property names (strings)
                      and value being property values (strings or nested properties group).
                      Can be None.

    Returns Type.PropertiesGroup, or None.
    """

    if not properties:
        return None

    ret = Type.PropertiesGroup()
    ret.name = ''

    for key, value in properties.items():
        if isinstance(value, six.string_types):
            # prop = Type.Property()
            prop = ret.properties.add()
            prop.name = key
            prop.value = value
        elif isinstance(value, list):
            for item in value:
                if isinstance(item, six.string_types):
                    child = ret.properties.add()
                    child.name = key
                    child.value = item
                else:
                    child = _to_property_group(item)
                    child.name = key
                    ret.children.extend([child])
        else:
            child = _to_property_group(value)
            if child:
                child.name = key
                ret.children.extend([child])

    return ret


def _to_params_list(parameters):
    """Converts the given parameters to a list of Type.Param.

    Arguments:
    parameters -- a dict with keys being parameter names and values being either parameter values, or
                  a dict with two keys: 'value', which is the parameter value, and 'children', which is a dict
                  of the same type as above. Example:
                  {'param1': 'value1', 'param2': {'value': 'value2', 'children': {'child_param1': 'value3'}}}
                  Can be None.

    Returns a list of Type.Param, or None.
    """

    if not parameters:
        return None

    ret = []

    for key, value in parameters.items():
        param = Type.Param()
        param.name = key
        if isinstance(value, ParameterFile):
            param.value = value.uri
        elif isinstance(value, ResponseMapFile):
            param.value = value.uri
        elif isinstance(value, dict):
            param.value = str(value['value'])
            if 'children' in value:
                param.parameters.extend(_to_params_list(value['children']))
        else:
            param.value = str(value)

        ret.append(param)

    return ret


class SessionProfileInformation(object):
    """ A details object to specify agent parameters"""

    def __init__(self, agent_name, agent_id, protocol_version, agent_type, capabilities):
        """Initialise a agent parameters object
        agent_name -- a name of agent.
        agent_id -- a identifier of agent.
        protocol_version -- a version of protocol used
        """
        self.name = agent_id
        self.agent_name = agent_name
        self.protocol_version = protocol_version
        self.agent_type = agent_type
        self.capabilities = {}

        for cap in capabilities:
            cur_cap = self.capabilities.get(cap.name)
            if cur_cap is None:
                cur_cap = cap.value
            elif type(cur_cap) == list:
                cur_cap.append(cap.value)
            else:
                cur_cap = [cur_cap, cap.value]
            self.capabilities[cap.name] = cur_cap

    def __str__(self):
        return str(vars(self))


class SessionProfile(UriIdentity):
    """Session profile representation.

    Allows to open session.

    Recommended way to use this class is as follows:

        with proj.session_name_ffsp.open() as s1:
            # invoke a 'init_routes' quickcall
            response = s1.init_routes(interface='Eth2/2', timeout=150)

            # other syntax of same command
            response = s1.init_routes('-interface Eth2/2 -timeout 150')

            # invoke a command:
            response = s1.command('command', response_map=proj.response_map_ffrm)

    Constructing instances of this class directly may be handy for testing, but generally is discouraged.

    Properties:
    agent -- agent information
    """

    def __init__(self, protocol_socket, agent_type, uri, dependencies=None):
        """Initiates a new session profile.

        Arguments:
        protocol_socket -- an instance of ProtocolSocket. It must be connected by the time open() is called.
        uri -- session profile URI, e.g. project://my_project/session_profiles/slc_test.ffsp
        dependencies -- session profile dependencies,
                        e.g. ['file:///home/dsavenko/itest/itest/dev/src/non-plugins/SpirentSLC/my_project.itar']

        """
        UriIdentity.__init__(self, uri)
        self._protosock = protocol_socket
        self._deps = dependencies

        self.agent = SessionProfileInformation(self._protosock.agent_name, self._protosock.agent_id,
                                               self._protosock.protocol_version, agent_type,
                                               self._protosock.capabilities)

    def open(self, parameter_file=None, agent_requirements=None, properties=None, response_map_lib=None,
             response_cache_size=10 * 1024 * 1024, **kwargs):
        """Opens the session.

        Arguments:
        parameter_file -- a single parameter file (URI), or a list of parameter files.
        agent_requirements -- dict of the format {'requirement_name': 'requirement_value'}
        properties -- a dictionary with keys being property names (strings)
                      and value being property values (strings or nested properties group).

        Raises SLCError, if socket is close. May raise socket.error.
        """
        if not self._protosock.is_open():
            raise SLCError('SLC connection is closed')

        return SlcSession(self._protosock, self.agent, self.uri, self._deps, response_cache_size, parameter_file,
                          agent_requirements, properties, response_map_lib, **kwargs)

    def list(self, qc_name=None):
        """Returns all QuickCalls available on a given session profile"""

        if qc_name:
            resp = self._protosock.query_session(self._uri, qc_name)
            qc = resp.quickCall[0]
            return dict((arg.name, arg.description) for arg in qc.args)

        quickcalls = dict()
        resp = self._protosock.query_session(self._uri)
        for qc in resp.quickCall:
            quickcalls[qc.name] = dict((arg.name, arg.description) for arg in qc.args)

        return quickcalls

    def __dir__(self):
        """ return a list of methods available to session profile"""
        return dir(super) + list(self.list().keys())

    def __getattr__(self, key):
        ret = self[key]
        if not ret:
            raise AttributeError(key)
        return ret


class SlcSession(UriIdentity):

    def __init__(self, protocol_socket, agent_type, uri, dependencies=None, response_cache_size=10 * 1024 * 1024,
                 parameter_file=None, agent_requirements=None, properties=None, response_map_lib=None, **kwargs):
        """Initiates a new session.

        Arguments:
        protocol_socket -- an instance of ProtocolSocket. It must be connected by the time open() is called.
        uri -- session profile URI, e.g. project://my_project/session_profiles/slc_test.ffsp
        dependencies -- session profile dependencies,
                        e.g. ['file:///home/dsavenko/itest/itest/dev/src/non-plugins/SpirentSLC/my_project.itar']

        """
        UriIdentity.__init__(self, uri)
        self._protosock = protocol_socket
        self._deps = dependencies

        self.agent = SessionProfileInformation(self._protosock.agent_name, self._protosock.agent_id,
                                               self._protosock.protocol_version, agent_type,
                                               self._protosock.capabilities)
        self.session_id = None
        self.open_response = None
        self._response_listener = SessionActionListener()
        self._response_storage = ResponseStorage(response_cache_size) if response_cache_size > 0 else None
        self._open(parameter_file, agent_requirements, properties, response_map_lib, **kwargs)

    def _open(self, parameter_file=None, agent_requirements=None, properties=None, response_map_lib=None, **kwargs):
        """Opens the session.

        Arguments:
        parameter_file -- a single parameter file (URI), or a list of parameter files.
        agent_requirements -- dict of the format {'requirement_name': 'requirement_value'}
        properties -- a dictionary with keys being property names (strings)
                      and value being property values (strings or nested properties group).

        Raises ValueError, if the session is already opened. May raise socket.error.
        """

        if self.session_id:
            raise ValueError('Session already opened with id=' + self.session_id)

        if properties is None:
            properties = {}
            for key, value in kwargs.items():
                properties[key] = str(value)

        # Convert parameter file to uri
        if isinstance(parameter_file, ParameterFile):
            parameter_file = parameter_file.uri

        # Convert project to uri
        if isinstance(response_map_lib, UriIdentity):
            response_map_lib = response_map_lib.uri

        if response_map_lib != None and not isinstance(response_map_lib, six.string_types):
            raise ValueError('Wrong response_map_lib value is passed. Should be URI' + str(response_map_lib))

        rsp = self._protosock.start_session(self._uri,
                                            dependencies=self._deps,
                                            param_files=_to_list(parameter_file),
                                            requirements=_to_requirements(agent_requirements),
                                            property_group=_to_property_group(properties),
                                            resp_map_lib=response_map_lib)
        self.session_id = rsp.sessionId
        self.open_response = SessionActionResponse(rsp)
        return self

    def close(self):
        """Closes the session.

        Does nothing, if the session has not been opened yet. May raise socket.error.
        """
        session_id = self.session_id
        self.session_id = None
        self.open_response = None
        if self._response_storage:
            self._response_storage.close()
        if session_id:
            self._protosock.close_session(session_id)

    def is_open(self):
        """ Return if session is stil opened or not"""
        return self.session_id != None and self._protosock != None and self._protosock.is_open()

    def session_properties(self):
        """Query a running session for all session properties"""
        return self.invoke_action('session_properties')

    def session_attribute(self, attr_name, default=None):
        """Query a running session for all session attributes"""
        return self.invoke_action('__session_attributes__' + attr_name, default)

    def step_properties(self, action, command=None, parameters=None, properties=None, response_map=None):
        """Query a running session for step actin properties"""
        return self.invoke_action('step_properties_' + action, command=command, parameters=parameters,
                                  properties=properties, response_map=response_map)

    def invoke_action(self, action, command=None, parameters=None, response_map=None, properties=None):
        """Invokes session action or quickcall.

        Generally, you do not need to use this method directly. E.g. instead of calling

            ssh_session.invoke_action('command', 'ls')

        you can just call

            ssh_session.command('ls')

        For Quickcalls, instead of calling

            session.invoke_action('my_quickcall', parameters={'param1': 'value1', 'param2': 123})

        you can just call

            session.my_quickcall(param1='value1', param2=123)

        Arguments:
        action -- session action or QuickCall name. Depends on a session.
                  Widely used actions: 'close' to close the session, 'command' to execute a specific session command.
        command -- command to execute
        parameters -- a dict with keys being parameter names and values being either parameter values, or
                      a dict with two keys: 'value', which is the parameter value, and 'children', which is a dict
                      of the same type as above. Example:
                      {'param1': 'value1', 'param2': {'value': 'value2', 'children': {'child_param1': 'value3'}}}
        properties -- a dictionary with keys being property names (strings)
                      and value being property values (strings or nested properties group).
                      A {'param_group.param': 'value1'} syntax could be used to update nested values.
        responseMap -- URI of the response map file to use.

        Returns action response. Console calling is equivalent to response._text.
        Raises ValueError, if the session is not opened. May raise socket.error
        """

        if not self.session_id:
            raise ValueError('Session is not opened')

        # Convert response map to it uri
        if isinstance(response_map, ResponseMapFile):
            response_map = response_map.uri

        def is_nested_response(msg):
            return msg.type == msg.SESSION_ACTION_RESPONSE and \
                   msg.sessionActionResponse.HasField("nestedActionDetails")

        def is_execution_issue(msg):
            return msg.type == msg.EXECUTION_ISSUE_MESSAGE

        def parent_id(step_id):
            id_parts = step_id.split('.')
            result = '.'.join(id_parts[:-1])
            return result if '.' in result else ''

        context = StepExecutionContext(self.session_id, self._protosock)
        root_builder = _ResponseBuilder(None)
        collected_steps = {root_builder.id: root_builder}

        orphaned_issues = {}

        def nested_step_collector(msg):
            if not is_nested_response(msg):
                # Check if it is execution_issue and collect it
                if is_execution_issue(msg):
                    msg = msg.executionIssueMessage
                    step_id = msg.stepId
                    if step_id == '':
                        root_builder.add_issue(msg)
                    elif step_id in collected_steps and isinstance(collected_steps[step_id], _ResponseBuilder):
                        # step is completed and step is under construction
                        builder = collected_steps[step_id]
                        builder.add_issue(msg)
                    else:
                        if step_id in orphaned_issues:
                            orphaned_issues[step_id].append(msg)
                        else:
                            orphaned_issues[step_id] = [msg]
                return
            response_message = msg.sessionActionResponse
            nested_details = response_message.nestedActionDetails
            step_id = nested_details.stepId
            if step_id == '':
                # 1st step is a Builder, skip it
                return
            if nested_details.status == Type.SessionActionUpdateStatus.Value('STARTED'):
                if step_id not in collected_steps:
                    # we see the step for the first time
                    collected_steps[step_id] = _ResponseBuilder(response_message)
                    self.response_listener.process_start(context, response_message)
                else:
                    # this is just an update of the step that we've already seen
                    builder = collected_steps[step_id]
                    builder.update_response(response_message)
                    self.response_listener.process_update(context, response_message)
            elif step_id in collected_steps and isinstance(collected_steps[step_id], _ResponseBuilder):
                # step is completed and step is under construction
                builder = collected_steps[step_id]
                builder.update_response(response_message)
                response = self._cache(step_id, builder.build())
                collected_steps[step_id] = response
                parent = collected_steps[parent_id(step_id)]
                parent.add_nested(response)
                self.response_listener.process_done(context, response)

                if step_id in orphaned_issues:
                    issues = orphaned_issues.pop(step_id)
                    for issue in issues:
                        builder.add_issue(issue)
            else:
                # step is completed but it wasn't seen before or
                # step is seen before but need to be overwritten
                issues = []
                if step_id in orphaned_issues:
                    issues = orphaned_issues.pop(step_id)
                response = self._cache(step_id, SessionActionResponse(response_message, execution_issues=issues))
                collected_steps[step_id] = response
                parent = collected_steps[parent_id(step_id)]
                parent.add_nested(response)
                self.response_listener.process_done(context, response)

        command = command if (command is None) else str(command)
        params_list = _to_params_list(parameters)
        property_group = _to_property_group

        resp = self._protosock.invoke_action(self.session_id,
                                             action=action,
                                             command=command,
                                             params=params_list,
                                             response_map=response_map,
                                             property_group=property_group(properties),
                                             nested_step_collector=nested_step_collector)

        root_builder.update_response(resp)
        root_builder.update_action(session_id=self.session_id,
                                   action=action,
                                   command=command,
                                   params=params_list,
                                   response_map=response_map,
                                   property_group=property_group(properties))

        return root_builder.build()

    def _cache(self, step_id, response):
        if self._is_caching_disabled():
            return response
        self._response_storage[step_id] = response
        return _CachedSessionResponse(step_id, self._response_storage, response.nested_steps, response._execution_issues)

    def _is_caching_disabled(self):
        return self._response_storage is None

    def list(self, qc_name=None):
        """Returns all QuickCalls available on a given session profile"""

        if qc_name:
            resp = self._protosock.query_session(self._uri, qc_name)
            qc = resp.quickCall[0]
            return dict((arg.name, arg.description) for arg in qc.args)

        quickcalls = dict()
        resp = self._protosock.query_session(self._uri)
        for qc in resp.quickCall:
            quickcalls[qc.name] = dict((arg.name, arg.description) for arg in qc.args)

        return quickcalls

    @property
    def response_listener(self):
        return self._response_listener

    @response_listener.setter
    def response_listener(self, listener):
        if listener is None:
            listener = SessionActionListener()
        self._response_listener = listener

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()

    def __dir__(self):
        """ return a list of methods available to project"""
        return dir(super) + list(self.list().keys())

    def __getitem__(self, key):
        return lambda *args, **kwargs: self.invoke_action(key, *args, **_to_invoke_args(kwargs))

    def __getattr__(self, key):
        ret = self[key]
        if not ret:
            raise AttributeError(key)
        return ret


class _ResponseBuilder(object):
    def __init__(self, response_message):
        self._response_message = response_message
        self._execution_issues = []
        self._nested_steps = {}

    def add_nested(self, response):
        self._nested_steps[response.id] = response

    def add_issue(self, issue):
        self._execution_issues.append(issue)

    def update_response(self, response_message):
        self._response_message = response_message

    def update_action(self, session_id, action, command=None, params=None, response_map=None, property_group=None):

        message = create_action_message(session_id=session_id,
                                        action=action,
                                        command=command,
                                        params=params,
                                        response_map=response_map,
                                        property_group=property_group)

        self._response_message.nestedActionDetails.action.CopyFrom(message)

    @property
    def id(self):
        if self._response_message is None:
            return ''
        return self._response_message.nestedActionDetails.stepId

    def build(self):
        built_steps = self._nested_steps.values()
        nested_value = sorted(built_steps, key=cmp_to_key(response_step_compare))
        return SessionActionResponse(self._response_message,
                                     nested_steps=nested_value,
                                     execution_issues=self._execution_issues)

    def __repr__(self):
        if self._response_message is None:
            return 'None'
        return repr(self._response_message.nestedActionDetails.stepId)


class SessionActionListener:
    def __init__(self):
        pass

    def process_start(self, context, response_message):
        pass

    def process_update(self, context, response_message):
        pass

    def process_done(self, context, response):
        pass


class StepExecutionContext(object):
    def __init__(self, session_id, protocol_socket):
        self._session_id = session_id
        self._protocol_socket = protocol_socket

    def cancel(self):
        self._protocol_socket.cancel_step(self._session_id)


class _CachedSessionResponse(object):
    def __init__(self, step_id, cache, nested_steps=(), execution_issues=[]):
        self._step_id = step_id
        self._cache = cache
        self._nested_steps = tuple(nested_steps)
        self._execution_issues = execution_issues

    @property
    def nested_steps(self):
        return self._nested_steps

    @property
    def id(self):
        return self._step_id

    @property
    def issues(self):
        return self._execution_issues


    def add_nested(self, response):
        updated_steps = [step for step in self._nested_steps if step.id != response.id] + [response]
        self._nested_steps = tuple(sorted(updated_steps, key=cmp_to_key(response_step_compare)))

    def add_issue(self, issue):
        self._execution_issues.append(issue)

    def __getattr__(self, key):
        response = self._cache[self._step_id]
        return getattr(response, key)

    def __repr__(self):
        response = self._cache[self._step_id]
        return repr(response)



# ****************       BEGIN-STANDARD-COPYRIGHT      ***************
#
# Copyright (c) 2018, Spirent Communications.
#
# All rights reserved. Proprietary and confidential information of Spirent Communications.
#
#  ***************        END-STANDARD-COPYRIGHT       ***************
"""SLC API that facilitates client code organization"""
import os
import re
import sys
import json
import inspect

from .exception import SLCError
from io import StringIO
from io import BytesIO
from .SLC import get_instance

from SpirentSLC.yaml_execution.operations import *

class Status:
    """A class to represent status of a test case being executed.

    Test status could be in one of the following states:
    Status.PASSED - test case passed
    Status.FAIL - test failed (e.g. because certain assertion criteria was not meet)
    Status.ABORTED - test case was aborted
    Status.CANCELED - test case was canceled
    Status.INDETERMINATE - test result is indeterminate

    Example:
        from SpirentSLC.Execution import Status
        status = Status()
        status.passTestIfNotAlreadyFailed()
        if status.get() == Status.PASSED:
            print "test passed!"
    """

    PASSED = 'PASSED'
    FAILED = 'FAILED'
    ABORTED = 'ABORTED'
    CANCELLED = 'CANCELLED'
    INDETERMINATE = 'INDETERMINATE'

    def __init__(self, state=INDETERMINATE):
        self._state = state

    def pass_test(self, log=None):
        """Pass test whatever current status is"""
        self._state = Status.PASSED
        if log:
            log.info("Test case has passed.")

    def fail_test(self, log=None):
        """Fail test whatever current status is"""
        self._state = Status.FAILED
        if log:
            log.error("Test case has failed.")

    def fail_test_if_passing(self, log=None):
        """
            Fail test if current status is not already FAILED or ABORTED.
            Otherwise status will remain unchanged.
        """
        if self._state != Status.FAILED and self._state != Status.ABORTED:
            self._state = Status.FAILED
            if log:
                log.error("Test case has failed.")

    def pass_test_if_not_already_failed(self, log=None):
        """
            Pass test if current state is either INDETERMINATE or PASSED.
            Otherwise status will remain unchanged.
        """
        if self._state == Status.INDETERMINATE or self._state == Status.PASSED:
            self._state = Status.PASSED
            if log:
                log.info("Test case has passed.")

    def abort_test(self, log=None):
        """Change test status to ABORTED"""
        self._state = Status.ABORTED
        if log:
            log.error("Test case has aborted.")

    def cancel_test(self, log=None):
        """Change test status to CANCELLED"""
        self._state = Status.CANCELLED
        if log:
            log.error("Test case has been cancelled.")

    def get(self):
        """Get current status (could be Status.FAILED, Status.PASSED, etc.)"""
        return self._state


class TestTermination(Exception):
    """Exception indicating test case was terminated abruptly"""
    pass


def _get_json_paths(paths):
    """
    Transform json delete paths string into paths list.

    Arguments:
        paths - string of the form '{"some/path", "other/path", ... }'
    """
    as_json_list = '[' + paths.strip()[1:-1] + ']'
    return json.loads(as_json_list)


def add_json_nodes(target, path_value):
    """
    Extend target by value at the given path.

    Arguments:
        target - json strings
        path_value - string with the same format as in 'json' step of iTest.

    Example:
        target = '{"x": [{"y":42}]}'
        add_json_node(target, '{ "x/[0]/" : {"z":true} }')
        The result will be '{"x": [{"y":42, "z":true}]}'

    See iTest Help for details on 'json' step format.
    """
    target = json.loads(target)
    for path, value in json.loads(path_value).items():
        path = path.strip()
        if path == "/":
            target.update(value)
            return json.dumps(target)
        node, last_key = _locate(target, path)
        if isinstance(node, list):
            if path.endswith("/"):
                node[last_key].update(value)
            elif isinstance(value, list):
                node[last_key:last_key] = value
            else:
                if last_key < len(node):
                    node.insert(last_key, value)
                else:
                    node.append(value)
        elif isinstance(node, dict):
            def_value = {} if isinstance(value, dict) else []
            cur_value = node.setdefault(last_key, def_value)
            if isinstance(cur_value, dict):
                cur_value.update(value)
            elif isinstance(cur_value, list):
                cur_value.append(value)
            else:
                raise Exception("Unexpected type of %s element in %s dictionary. Expect 'list' or 'dict' but %s provided" % (str(last_key), str(node), type(cur_value)))
    return json.dumps(target)


def set_json_nodes(target, path_value):
    """
    Set value of target at the given path.

    Arguments:
        target - json string
        path_value - string with the same format as in 'json' step of iTest.

    Example:
        target = '{"x": [0, 1, 2]}'
        set_json_node(target, '{ "x/[1]" : {"y":true} }')
        The result will be '{"x": [0, {"y":true}, 2]}'

    See iTest Help for details on 'json' step format.
    """
    target = json.loads(target)
    for path, value in json.loads(path_value).items():
        node, last_key = _locate(target, path)
        node[last_key] = value
    return json.dumps(target)


def delete_json_nodes(target, paths):
    """
    Extend target by value at the given path.

    Arguments:
        target - json string
        paths - string with the same format as in 'json' step of iTest.

    Example:
        target = '{"x": [5, 6, 7]}'
        delete_json_node(target, '{"x/[1]"}')
        The result will be '{"x": [5, 7]}'

    See iTest Help for details on 'json' step format.
    """
    target = json.loads(target)
    for path in _get_json_paths(paths):
        node, last_key = _locate(target, path)
        del node[last_key]
    return json.dumps(target)


def get_json_nodes(target, paths):
    """
    Query json object fields.

    Arguments:
        target - json string from which fields will be extracted
        paths - string with the same format as in 'json' step of iTest.
    """
    target = json.loads(target)
    for path in _get_json_paths(paths):
        path = path.strip()
        keys = list(map(_to_key, path.split("/")))
        value = _find_child(target, keys)
        last_object_key = _find_last_object_key(target, keys)
        return json.dumps({last_object_key: value})


def _find_last_object_key(target, keys):
    node = target
    result = None
    for key in keys:
        if isinstance(node, dict):
            result = key
        node = node[key]
    return result


def _locate(target, path):
    path = path.strip()
    keys = list(map(_to_key, path.split("/")))
    last_key = keys.pop()
    if not isinstance(last_key, int):
        last_key = last_key or keys.pop()
    node = _find_child(target, keys)
    return (node, last_key)


def _to_key(path_element):
    if path_element.startswith("[") and path_element.endswith("]"):
        return int(path_element[1:-1])
    return path_element


def _find_child(node, keys):
    for key in keys:
        node = node[key]
    return node


class Params:
    """Provide API to access test case parameters"""

    def __init__(self, params=None, read_command_line=True):
        self._params = params or dict()
        if read_command_line:
            self._read_command_line_params()

    def __call__(self, path, default_value=None):
        key = path.replace("/", ".")
        return self._params.get(key) if key in self._params else default_value

    def _read_command_line_params(self):
        command_line_params = self._parse_command_line_params()
        for key, value in command_line_params.items():
            self._params[key] = value  # override

    def _parse_command_line_params(self):
        result = {}
        remain = list(reversed(sys.argv[1:]))
        while remain:
            name = remain.pop()
            if name.startswith("-") and remain:
                name = name[1:]
                value = self._parse_value(remain.pop())
                if name in result and type(result[name]) is list:
                    result[name].append(value)
                elif name in result:
                    result[name] = [result[name], value]
                else:
                    result[name] = value
        return result

    def _parse_value(self, command_line_value):
        return command_line_value


def gget(name, default_value=None):
    return globals().get(name, default_value)


def gset(name, value):
    globals()[name] = value


def char(char_code):
    return str(get_instance().char_command(char_code))


def jsonSelect(json_string, query_xpath):
    return get_instance().jsonselect_command(json_string, query_xpath)


def store_session(session_name, session):
    get_instance()._sessions[session_name] = session


def profile(session_name, attr_name, default=None):
    try:
        session = get_instance()._sessions[session_name]
    except KeyError:
        raise KeyError("Session {} is not found, available sessions: {} ".format(session_name, get_instance()._sessions.keys()))
    return session.session_attribute(attr_name, default)


def store_response(var_name, raw_value):
    get_instance()._responses[var_name] = raw_value


def query(var_name, mapper_query, always_list=None):
    val = get_instance().cached_response(var_name)
    return val.query(mapper_query)


def response(var_name, always_list=None, regex=None, group=None):
    val = get_instance()._responses[var_name]
    return val.text


def velocity(subcommand, *argv):
    return get_instance().velocity_command(subcommand, *argv)


def xpathEval(query):
    return get_instance().xpath_eval_command(query)


def info(subcommand, *argv):
    unsupported_subcommands = ['testCaseProjectPath', 'testCaseProject', 'threadID', 'testReportID', 'step']
    if subcommand in unsupported_subcommands:
        raise NotImplementedError('Command info %s is not supported' % subcommand)
    # Execute some info commands on the local side
    if subcommand == 'exists':
        if len(argv) < 2:
            raise ValueError('Invalid argument received. Usage: info exists local|global|param <varName>')
        frame = inspect.currentframe()
        scope, var_name = argv[0], argv[1]
        result = False
        try:
            if scope == 'local':
                result = var_name in frame.f_back.f_locals
            elif scope == 'global':
                result = var_name in frame.f_back.f_globals
            elif scope == 'param':
                try:
                    result = var_name in frame.f_back.f_locals['param']._params
                except KeyError:
                    result = False
        finally:
            del frame
        return get_instance().response(str(result))
    if subcommand == 'procedure':
        frame = inspect.currentframe()
        try:
            return get_instance().response(frame.f_back.f_code.co_name)
        finally:
            del frame
    if subcommand == 'testCaseName':
        return get_instance().response(os.path.basename(sys.argv[0]))
    if subcommand == 'testCaseFile':
        return get_instance().response(sys.argv[0])
    return get_instance().info_command(subcommand, *argv)


def file_pathToUri(path):
    return get_instance().file_path_to_uri_command(path)


def file_uriToPath(uri):
    return get_instance().file_uri_to_path_command(uri)


def internal_tbml(uri, subcommand, *argv):
    return get_instance().tbml_command(uri, subcommand, *argv)


def quote(uri):
    return get_instance().quote(uri)


def internal_query(response_or_name , xpath, xpath_version):
    if isinstance(response_or_name, str):
        response = get_instance().cached_response(response_or_name)
    else:
        response = response_or_name
    if xpath_version == "1.0":
        return response.query(xpath)
    elif xpath_version == "3.1":
        # Communicate with agent to evaluate 3.1 queries
        matches_response = get_instance().query31_command(response.xml_string, xpath, response.aliases)
        result_list = json.loads(str(matches_response))
        return result_list[0] if len(result_list) == 1 else result_list
    raise ValueError('Unknown XPath version: ' + xpath_version)

def yamlCreate(file_uri=None, content=''):
    if file_uri is not None:
        with open(file_uri, mode='r', encoding='UTF-8') as f:
            return f.read()
    return content


def yamlGet(source, document_index=0, path=None):
    return YamlGetOperation(YamlQuery(document_index, path)).execute(source)


def yamlDelete(source, document_index=0, path=None):
    return YamlDeleteOperation(YamlQuery(document_index, path)).execute(source)


def yamlAdd(source, value, document_index=0, path=None):
    return YamlAddOperation(YamlQuery(document_index, path), value).execute(source)


def yamlSet(source, value, document_index=0, path=None):
    return YamlSetOperation(YamlQuery(document_index, path), value).execute(source)


def _action_type(action):
    if action.action != "action":
        return None
    return action.description.split(' ')[0]


_EXECUTION_ISSUE_PATTERN = re.compile(r'DeclareExecutionIssue\s(\w+)\s*:(.*)')


def _handle_execution_issue(action, status, logger):
    matched = _EXECUTION_ISSUE_PATTERN.match(action.description)
    if not matched:
        # unexpected execution issue format
        return
    severity = matched.group(1)
    message = matched.group(2)
    if severity == "OK" or severity == "Information":
        logger.info(message)
    elif severity == "Error":
        logger.error(message)
    elif severity == "Warning":
        logger.warn(message)


def _handle_action(action, status, logger):
    action_type = _action_type(action)
    if action_type == "DeclareExecutionIssue":
        _handle_execution_issue(action, status, logger)
    elif action_type == "FailTest":
        status.fail_test(logger)

    for child_action in action.actions:
        _handle_action(child_action, status, logger)


def handle_step_results(response, status, logger):
    """
    Handle actions affecting test output (exeuction issues) or its status.
    """
    for action in response.message.details.postProcessing:
        _handle_action(action, status, logger)


class StdoutCapture:
    def __init__(self):
        self.__default_stdout = sys.stdout

    def __enter__(self):
        self.__default_stdout = sys.stdout

        # Check python version
        if sys.version_info[0] >= 3:
            sys.stdout = StringIO()
        else:
            sys.stdout = BytesIO()
        return self

    def capture(self):
        if sys.version_info[0] >= 3:
            result = sys.stdout.getvalue().strip()
            sys.stdout.truncate()
            return result
        else:
            sys.stdout.seek(0)
            result = sys.stdout.read().decode('utf-8').strip()
            sys.stdout.truncate()
            return result

    def __exit__(self, type, value, traceback):
        sys.stdout = self.__default_stdout





# ****************       BEGIN-STANDARD-COPYRIGHT      ***************
#
# Copyright (c) 2017, Spirent Communications.
#
# All rights reserved. Proprietary and confidential information of Spirent Communications.
#
#  ***************        END-STANDARD-COPYRIGHT       ***************

import warnings
from zipfile import ZipFile, ZIP_STORED


class _QueueItem(object):
    def __init__(self, value, queue):
        self.value = value
        self.prev = None
        self.next = None
        self._queue = queue

    @property
    def queue(self):
        return self._queue

    def __repr__(self):
        return repr(self.value)


class _Queue(object):
    def __init__(self):
        self._first = None
        self._last = None
        self._size = 0

    @staticmethod
    def _attach(first, second):
        if first is not None:
            first.next = second
        if second is not None:
            second.prev = first

    def _detach(self, item):
        if item.prev is not None:
            item.prev.next = item.next
        if item.next is not None:
            item.next.prev = item.prev
        if self._first == item:
            self._first = item.next
        if self._last == item:
            self._last = item.prev
        item.next = None
        item.prev = None
        return item

    def push_last(self, value):
        item = _QueueItem(value, self)
        self._attach(self._last, item)
        self._last = item
        self._first = self._first or item
        self._size += 1
        return item

    def pop_first(self):
        if not self:
            raise IndexError('pop_first from empty queue')
        self._size -= 1
        return self._detach(self._first).value

    def shift_last(self, item):
        if item.queue is not self:
            raise ValueError('shift foreign item')
        self._detach(item)
        self._attach(self._last, item)
        self._last = item
        self._first = self._first or item

    def remove(self, item):
        if item.queue is not self:
            raise ValueError('removing foreign item')
        self._detach(item)
        self.size -= 1
        return item

    def __len__(self):
        return self._size


class LRUCache(object):
    def __init__(self, storage, max_size):
        self._storage = storage
        self._max_size = max_size
        self._queue = _Queue()
        self._index = {}
        self._total_item_size = 0

    def __contains__(self, key):
        return key in self._index

    def __getitem__(self, key):
        if key in self._index:
            item = self._index[key]
            self._queue.shift_last(item)
            key, value, size = item.value
            return value

        value, size = self._retrieve(key)

        self._index[key] = self._queue.push_last((key, value, size))
        self._total_item_size += size
        self._clear()
        return value

    def __delitem__(self, key):
        if key in self._index:
            item = self._index[key]
            self._queue.remove(item)
            del self._index[key]
            key, value, size = item.value
            self._total_item_size -= size

    def _retrieve(self, key):
        try:
            return self._storage[key]
        except KeyError:
            raise KeyError(key)

    def _clear(self):
        while self._total_item_size > self._max_size and len(self._index) > 1:
            key, value, size = self._queue.pop_first()
            del self._index[key]
            self._total_item_size -= size

    def __repr__(self):
        return repr(self._index)

    @property
    def size(self):
        return self._total_item_size

    def keys(self):
        return self._index.keys()


class ZipFileStorage(object):
    def __init__(self, stream, deserialize=lambda x: x, serialize=str):
        self._file = stream
        self._zipfile = ZipFile(self._file, mode='a', compression=ZIP_STORED)
        self._deserialize = deserialize
        self._serialize = serialize

    def __del__(self):
        self.close()

    def close(self):
        self._zipfile.close()

    def load(self, entry_name):
        entry = self._zipfile.getinfo(entry_name)
        data = self._zipfile.read(entry)
        size = len(data)
        value = self._deserialize(data)
        return value, size

    def __getitem__(self, key):
        return self.load(key)

    def store(self, entry_name, value):
        data = self._serialize(value)
        with warnings.catch_warnings():
            # we will overwrite some steps and get warning of entry key already in use
            warnings.simplefilter("ignore")
            self._zipfile.writestr(entry_name, data)
        return len(data)

    def __repr__(self):
        return repr(self._zipfile.namelist())
